@@ -111,9 +111,7 @@ do_iconview_edit (GtkWidget *do_widget)
       gtk_window_set_display (GTK_WINDOW (window),
                               gtk_widget_get_display (do_widget));
       gtk_window_set_title (GTK_WINDOW (window), "Editing and Drag-and-Drop");
-
-      g_signal_connect (window, "destroy",
-                        G_CALLBACK (gtk_widget_destroyed), &window);
+      g_object_add_weak_pointer (G_OBJECT (window), (gpointer *)&window);
 
       store = create_store ();
       fill_store (store);
@@ -146,13 +144,13 @@ do_iconview_edit (GtkWidget *do_widget)
                                       "text", COL_TEXT,
                                       NULL);
 
-      gtk_container_add (GTK_CONTAINER (window), icon_view);
+      gtk_window_set_child (GTK_WINDOW (window), icon_view);
     }
 
   if (!gtk_widget_get_visible (window))
     gtk_widget_show (window);
   else
-    gtk_widget_destroy (window);
+    gtk_window_destroy (GTK_WINDOW (window));
 
   return window;
 }
