@@ -57,12 +57,10 @@ do_overlay2 (GtkWidget *do_widget)
       gtk_text_iter_forward_word_end (&end);
       gtk_text_buffer_apply_tag (buffer, tag, &start, &end);
 
-      gtk_container_add (GTK_CONTAINER (window), overlay);
-      gtk_container_add (GTK_CONTAINER (overlay), sw);
-      gtk_container_add (GTK_CONTAINER (sw), text);
-
-      g_signal_connect (window, "destroy",
-                        G_CALLBACK (gtk_widget_destroyed), &window);
+      gtk_window_set_child (GTK_WINDOW (window), overlay);
+      gtk_overlay_set_child (GTK_OVERLAY (overlay), sw);
+      gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (sw), text);
+      g_object_add_weak_pointer (G_OBJECT (window), (gpointer *)&window);
 
       image = gtk_picture_new_for_resource ("/overlay2/decor1.png");
       gtk_overlay_add_overlay (GTK_OVERLAY (overlay), image);
@@ -97,7 +95,7 @@ do_overlay2 (GtkWidget *do_widget)
   if (!gtk_widget_get_visible (window))
     gtk_widget_show (window);
   else
-    gtk_widget_destroy (window);
+    gtk_window_destroy (GTK_WINDOW (window));
 
   return window;
 }
