@@ -1,4 +1,4 @@
-/* Spin Button
+/* Spin Buttons
  *
  * GtkSpinButton provides convenient ways to input data
  * that can be seen as a value in a range. The examples
@@ -196,8 +196,7 @@ do_spinbutton (GtkWidget *do_widget)
                             gtk_widget_get_display (do_widget));
     gtk_window_set_title (GTK_WINDOW (window), "Spin Buttons");
     gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
-    g_signal_connect (window, "destroy",
-                      G_CALLBACK (gtk_widget_destroyed), &window);
+    g_object_add_weak_pointer (G_OBJECT (window), (gpointer *)&window);
 
     adj = GTK_ADJUSTMENT (gtk_builder_get_object (builder, "basic_adjustment"));
     label = GTK_WIDGET (gtk_builder_get_object (builder, "basic_label"));
@@ -238,7 +237,7 @@ do_spinbutton (GtkWidget *do_widget)
   if (!gtk_widget_get_visible (window))
     gtk_widget_show (window);
   else
-    gtk_widget_destroy (window);
+    gtk_window_destroy (GTK_WINDOW (window));
 
   return window;
 }
