@@ -37,7 +37,7 @@ search_text_changed (GtkEntry         *entry,
     return;
 
   tab = gtk_stack_get_visible_child (GTK_STACK (win->stack));
-  view = gtk_bin_get_child (GTK_BIN (tab));
+  view = gtk_scrolled_window_get_child (GTK_SCROLLED_WINDOW (tab));
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
   /* Very simple-minded search implementation */
@@ -69,15 +69,15 @@ update_words (ExampleAppWindow *win)
   GtkWidget *tab, *view, *row;
   GtkTextBuffer *buffer;
   GtkTextIter start, end;
-  GList *children, *l;
   gchar *word, *key;
+  GtkWidget *child;
 
   tab = gtk_stack_get_visible_child (GTK_STACK (win->stack));
 
   if (tab == NULL)
     return;
 
-  view = gtk_bin_get_child (GTK_BIN (tab));
+  view = gtk_scrolled_window_get_child (GTK_SCROLLED_WINDOW (tab));
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
   strings = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
@@ -100,10 +100,8 @@ update_words (ExampleAppWindow *win)
     }
 
 done:
-  children = gtk_container_get_children (GTK_CONTAINER (win->words));
-  for (l = children; l; l = l->next)
-    gtk_container_remove (GTK_CONTAINER (win->words), GTK_WIDGET (l->data));
-  g_list_free (children);
+  while ((child = gtk_widget_get_first_child (win->words)))
+    gtk_list_box_remove (GTK_LIST_BOX (win->words), child);
 
   g_hash_table_iter_init (&iter, strings);
   while (g_hash_table_iter_next (&iter, (gpointer *)&key, NULL))
@@ -111,7 +109,7 @@ done:
       row = gtk_button_new_with_label (key);
       g_signal_connect (row, "clicked",
                         G_CALLBACK (find_word), win);
-      gtk_container_add (GTK_CONTAINER (win->words), row);
+      gtk_box_append (GTK_BOX (win->words), row);
     }
 
   g_hash_table_unref (strings);
@@ -130,7 +128,7 @@ update_lines (ExampleAppWindow *win)
   if (tab == NULL)
     return;
 
-  view = gtk_bin_get_child (GTK_BIN (tab));
+  view = gtk_scrolled_window_get_child (GTK_SCROLLED_WINDOW (tab));
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
   count = gtk_text_buffer_get_line_count (buffer);
@@ -266,7 +264,7 @@ example_app_window_open (ExampleAppWindow *win,
   view = gtk_text_view_new ();
   gtk_text_view_set_editable (GTK_TEXT_VIEW (view), FALSE);
   gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (view), FALSE);
-  gtk_container_add (GTK_CONTAINER (scrolled), view);
+  gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled), view);
   gtk_stack_add_titled (GTK_STACK (win->stack), scrolled, basename, basename);
 
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
