@@ -15,18 +15,18 @@ activate_cb (GtkApplication *app,
 
   search_bar = gtk_search_bar_new ();
   gtk_widget_set_valign (search_bar, GTK_ALIGN_START);
-  gtk_container_add (GTK_CONTAINER (window), search_bar);
+  gtk_window_set_child (GTK_WINDOW (window), search_bar);
   gtk_widget_show (search_bar);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_container_add (GTK_CONTAINER (search_bar), box);
+  gtk_search_bar_set_child (GTK_SEARCH_BAR (search_bar), box);
 
   entry = gtk_search_entry_new ();
   gtk_widget_set_hexpand (entry, TRUE);
-  gtk_container_add (GTK_CONTAINER (box), entry);
+  gtk_box_append (GTK_BOX (box), entry);
 
   menu_button = gtk_menu_button_new ();
-  gtk_container_add (GTK_CONTAINER (box), menu_button);
+  gtk_box_append (GTK_BOX (box), menu_button);
 
   gtk_search_bar_connect_entry (GTK_SEARCH_BAR (search_bar), GTK_EDITABLE (entry));
   gtk_search_bar_set_key_capture_widget (GTK_SEARCH_BAR (search_bar), window);
