@@ -238,8 +238,7 @@ client_handle_request (BroadwayClient *client,
                                      request->new_surface.x,
                                      request->new_surface.y,
                                      request->new_surface.width,
-                                     request->new_surface.height,
-                                     request->new_surface.is_temp);
+                                     request->new_surface.height);
       client->surfaces =
         g_list_prepend (client->surfaces,
                         GUINT_TO_POINTER (reply_new_surface.id));
@@ -513,7 +512,8 @@ incoming_client (GSocketService    *service,
   ev.base.time = broadway_server_get_last_seen_time (server);
   broadway_server_get_screen_size (server,
                                    &ev.screen_resize_notify.width,
-                                   &ev.screen_resize_notify.height);
+                                   &ev.screen_resize_notify.height,
+                                   &ev.screen_resize_notify.scale);
 
   broadway_events_got_input (&ev,
                              client->id);
