@@ -18,7 +18,7 @@ void            gdk_pre_parse                   (void);
 void            gdk_surface_freeze_toplevel_updates      (GdkSurface *surface);
 void            gdk_surface_thaw_toplevel_updates        (GdkSurface *surface);
 
-guint32         gdk_display_get_last_seen_time  (GdkDisplay *display);
+gboolean        gdk_surface_supports_edge_constraints    (GdkSurface *surface);
 
 void gdk_display_set_double_click_time     (GdkDisplay   *display,
                                             guint         msec);
