@@ -24,6 +24,7 @@
 
 #include <gdk/gdkversionmacros.h>
 #include <gdk/gdktypes.h>
+#include <gdk/gdkdevicetool.h>
 
 
 G_BEGIN_DECLS
@@ -68,25 +69,6 @@ typedef enum
   GDK_SOURCE_TABLET_PAD
 } GdkInputSource;
 
-/**
- * GdkInputMode:
- * @GDK_MODE_DISABLED: the device is disabled and will not report any events.
- * @GDK_MODE_SCREEN: the device is enabled. The device’s coordinate space
- *                   maps to the entire screen.
- * @GDK_MODE_SURFACE: the device is enabled. The device’s coordinate space
- *                   is mapped to a single surface. The manner in which this surface
- *                   is chosen is undefined, but it will typically be the same
- *                   way in which the focus surface for key events is determined.
- *
- * An enumeration that describes the mode of an input device.
- */
-typedef enum
-{
-  GDK_MODE_DISABLED,
-  GDK_MODE_SCREEN,
-  GDK_MODE_SURFACE
-} GdkInputMode;
-
 /**
  * GdkDeviceType:
  * @GDK_DEVICE_TYPE_MASTER: Device is a master (or virtual) device. There will
@@ -131,13 +113,7 @@ gboolean              gdk_device_get_has_cursor (GdkDevice *device);
 
 /* Functions to configure a device */
 GDK_AVAILABLE_IN_ALL
-GdkInputSource gdk_device_get_source    (GdkDevice      *device);
-
-GDK_AVAILABLE_IN_ALL
-GdkInputMode   gdk_device_get_mode      (GdkDevice      *device);
-GDK_AVAILABLE_IN_ALL
-gboolean       gdk_device_set_mode      (GdkDevice      *device,
-                                         GdkInputMode    mode);
+GdkInputSource gdk_device_get_source (GdkDevice    *device);
 
 GDK_AVAILABLE_IN_ALL
 gint           gdk_device_get_n_keys    (GdkDevice       *device);
@@ -184,11 +160,11 @@ void     gdk_device_free_history (GdkTimeCoord     **events,
 GDK_AVAILABLE_IN_ALL
 gint     gdk_device_get_n_axes     (GdkDevice       *device);
 GDK_AVAILABLE_IN_ALL
-GList *  gdk_device_list_axes      (GdkDevice       *device);
+char **  gdk_device_get_axis_names (GdkDevice       *device);
 GDK_AVAILABLE_IN_ALL
 gboolean gdk_device_get_axis_value (GdkDevice       *device,
                                     gdouble         *axes,
-                                    GdkAtom          axis_label,
+                                    const char      *axis_label,
                                     gdouble         *value);
 
 GDK_AVAILABLE_IN_ALL
@@ -221,6 +197,30 @@ GdkSeat     *gdk_device_get_seat            (GdkDevice *device);
 GDK_AVAILABLE_IN_ALL
 GdkAxisFlags gdk_device_get_axes            (GdkDevice *device);
 
+GDK_AVAILABLE_IN_ALL
+guint           gdk_device_get_num_touches  (GdkDevice *device);
+
+GDK_AVAILABLE_IN_ALL
+GdkDeviceTool * gdk_device_get_device_tool  (GdkDevice *device);
+
+GDK_AVAILABLE_IN_ALL
+gboolean        gdk_device_get_caps_lock_state (GdkDevice *device);
+
+GDK_AVAILABLE_IN_ALL
+gboolean        gdk_device_get_num_lock_state (GdkDevice *device);
+
+GDK_AVAILABLE_IN_ALL
+gboolean        gdk_device_get_scroll_lock_state (GdkDevice *device);
+
+GDK_AVAILABLE_IN_ALL
+GdkModifierType gdk_device_get_modifier_state (GdkDevice *device);
+
+GDK_AVAILABLE_IN_ALL
+PangoDirection  gdk_device_get_direction (GdkDevice *device);
+
+GDK_AVAILABLE_IN_ALL
+gboolean        gdk_device_has_bidi_layouts (GdkDevice *device);
+
 G_END_DECLS
 
 #endif /* __GDK_DEVICE_H__ */
