@@ -244,6 +244,10 @@ void                gdk_display_emit_opened           (GdkDisplay       *display
 void                gdk_display_setting_changed       (GdkDisplay       *display,
                                                        const char       *name);
 
+GdkEvent *          gdk_display_get_event             (GdkDisplay       *display);
+GdkEvent *          gdk_display_peek_event            (GdkDisplay       *display);
+gboolean            gdk_display_has_pending           (GdkDisplay       *display);
+
 
 G_END_DECLS
 
