@@ -76,8 +76,8 @@ GdkDrag *        gdk_drag_begin                (GdkSurface         *surface,
                                                 GdkDevice          *device,
                                                 GdkContentProvider *content,
                                                 GdkDragAction       actions,
-                                                gint                dx,
-                                                gint                dy);
+                                                double              dx,
+                                                double              dy);
 
 GDK_AVAILABLE_IN_ALL
 void            gdk_drag_drop_done   (GdkDrag  *drag,
@@ -91,6 +91,13 @@ void            gdk_drag_set_hotspot (GdkDrag *drag,
                                       gint     hot_x,
                                       gint     hot_y);
 
+GDK_AVAILABLE_IN_ALL
+GdkContentProvider *
+                gdk_drag_get_content (GdkDrag *drag);
+
+GDK_AVAILABLE_IN_ALL
+GdkSurface *    gdk_drag_get_surface (GdkDrag *drag);
+
 G_END_DECLS
 
 #endif /* __GDK_DND_H__ */
