@@ -31,10 +31,9 @@
  * @Short_description: Object representing an output
  *
  * GdkMonitor objects represent the individual outputs that are
- * associated with a #GdkDisplay. GdkDisplay has APIs to enumerate
- * monitors with gdk_display_get_n_monitors() and gdk_display_get_monitor(), and
- * to find particular monitors with gdk_display_get_primary_monitor() or
- * gdk_display_get_monitor_at_surface().
+ * associated with a #GdkDisplay. GdkDisplay keeps a #GListModel to enumerate
+ * and monitor monitors with gdk_display_get_monitors().  
+ * You can use gdk_display_get_monitor_at_surface() to find a particular monitor.
  */
 
 /**
@@ -531,47 +530,14 @@ gdk_monitor_set_connector (GdkMonitor *monitor,
 }
 
 void
-gdk_monitor_set_position (GdkMonitor *monitor,
-                          int         x,
-                          int         y)
+gdk_monitor_set_geometry (GdkMonitor *monitor,
+                          const GdkRectangle *geometry)
 {
-  g_object_freeze_notify (G_OBJECT (monitor));
+  if (gdk_rectangle_equal (&monitor->geometry, geometry))
+    return;
 
-  if (monitor->geometry.x != x)
-    {
-      monitor->geometry.x = x;
-      g_object_notify (G_OBJECT (monitor), "geometry");
-    }
-
-  if (monitor->geometry.y != y)
-    {
-      monitor->geometry.y = y;
-      g_object_notify (G_OBJECT (monitor), "geometry");
-    }
-
-  g_object_thaw_notify (G_OBJECT (monitor));
-}
-
-void
-gdk_monitor_set_size (GdkMonitor *monitor,
-                      int         width,
-                      int         height)
-{
-  g_object_freeze_notify (G_OBJECT (monitor));
-
-  if (monitor->geometry.width != width)
-    {
-      monitor->geometry.width = width;
-      g_object_notify (G_OBJECT (monitor), "geometry");
-    }
-
-  if (monitor->geometry.height != height)
-    {
-      monitor->geometry.height = height;
-      g_object_notify (G_OBJECT (monitor), "geometry");
-    }
-
-  g_object_thaw_notify (G_OBJECT (monitor));
+  monitor->geometry = *geometry;
+  g_object_notify (G_OBJECT (monitor), "geometry");
 }
 
 void
