@@ -24,6 +24,7 @@
 #error "Only <gdk/gdk.h> can be included directly."
 #endif
 
+#include <gdk/gdkseat.h>
 #include <gdk/gdksurface.h>
 #include <gdk/gdktoplevellayout.h>
 
@@ -87,6 +88,31 @@ void          gdk_toplevel_set_deletable         (GdkToplevel      *toplevel,
 GDK_AVAILABLE_IN_ALL
 gboolean      gdk_toplevel_supports_edge_constraints (GdkToplevel *toplevel);
 
+GDK_AVAILABLE_IN_ALL
+void          gdk_toplevel_inhibit_system_shortcuts  (GdkToplevel *toplevel,
+                                                      GdkEvent    *event);
+
+GDK_AVAILABLE_IN_ALL
+void          gdk_toplevel_restore_system_shortcuts  (GdkToplevel *toplevel);
+
+GDK_AVAILABLE_IN_ALL
+void          gdk_toplevel_begin_resize              (GdkToplevel    *toplevel,
+                                                      GdkSurfaceEdge  edge,
+                                                      GdkDevice      *device,
+                                                      int             button,
+                                                      double          x,
+                                                      double          y,
+                                                      guint32         timestamp);
+
+GDK_AVAILABLE_IN_ALL
+void          gdk_toplevel_begin_move                (GdkToplevel    *toplevel,
+                                                      GdkDevice      *device,
+                                                      int             button,
+                                                      double          x,
+                                                      double          y,
+                                                      guint32         timestamp);
+
+
 G_END_DECLS
 
 #endif /* __GDK_TOPLEVEL_H__ */
