@@ -101,7 +101,7 @@ gdk_vulkan_strerror (VkResult result)
    * encountered a newly added Vulkan error message.
    * You want to add it to this enum now.
    *
-   * Becuse the Vulkan people don't make adding this too easy, here's
+   * Because the Vulkan people don't make adding this too easy, here's
    * the process to manage it:
    * 1. go to
    *    https://github.com/KhronosGroup/Vulkan-Headers/blob/master/include/vulkan/vulkan_core.h
@@ -203,7 +203,7 @@ gdk_vulkan_strerror (VkResult result)
 #endif
 #if VK_HEADER_VERSION >= 105
     case VK_ERROR_FULL_SCREEN_EXCLUSIVE_MODE_LOST_EXT:
-      return "An operation on a swapchain created with VK_FULL_SCREEN_EXCLUSIVE_APPLICATION_CONTROLLED_EXT failed as it did not have exlusive full-screen access.";
+      return "An operation on a swapchain created with VK_FULL_SCREEN_EXCLUSIVE_APPLICATION_CONTROLLED_EXT failed as it did not have exclusive full-screen access.";
 #endif
 #if VK_HEADER_VERSION >= 131
     case VK_ERROR_UNKNOWN:
