@@ -31,7 +31,6 @@
 #include <limits.h>
 #include <errno.h>
 #include <sys/mman.h>
-#include <fribidi.h>
 
 #include "gdk.h"
 #include "gdkwayland.h"
@@ -412,6 +411,7 @@ update_direction (GdkWaylandKeymap *keymap)
        gint layouts, layout;
 
        layouts = xkb_keymap_num_layouts_for_key (keymap->xkb_keymap, key);
+       g_assert (layouts <= num_layouts);
        for (layout = 0; layout < layouts; layout++)
          {
            const xkb_keysym_t *syms;
