@@ -18,11 +18,13 @@
  */
 
 #include "gdkdisplayprivate.h"
-#include "gdkglcontext-win32.h"
 
 #ifndef __GDK_DISPLAY__WIN32_H__
 #define __GDK_DISPLAY__WIN32_H__
 
+#include "gdkwin32screen.h"
+#include "gdkwin32cursor.h"
+
 /* Define values used to set DPI-awareness */
 typedef enum _GdkWin32ProcessDpiAwareness {
   PROCESS_DPI_UNAWARE = 0,
@@ -30,19 +32,16 @@ typedef enum _GdkWin32ProcessDpiAwareness {
   PROCESS_PER_MONITOR_DPI_AWARE = 2
 } GdkWin32ProcessDpiAwareness;
 
-/* Define values for GL type used */
-typedef enum _GdkWin32GLContextType
-{
-  GDK_WIN32_GL_PENDING,
-  GDK_WIN32_GL_NONE,
-  GDK_WIN32_GL_WGL,
-  GDK_WIN32_GL_EGL
-} GdkWin32GLContextType;
+typedef enum _GdkWin32MonitorDpiType { 
+  MDT_EFFECTIVE_DPI  = 0,
+  MDT_ANGULAR_DPI    = 1,
+  MDT_RAW_DPI        = 2,
+  MDT_DEFAULT        = MDT_EFFECTIVE_DPI
+} GdkWin32MonitorDpiType;
 
 /* APIs from shcore.dll */
 typedef HRESULT (WINAPI *funcSetProcessDpiAwareness) (GdkWin32ProcessDpiAwareness value);
-typedef HRESULT (WINAPI *funcGetProcessDpiAwareness) (HANDLE                       handle,
-                                                      GdkWin32ProcessDpiAwareness *awareness);
+typedef HRESULT (WINAPI *funcGetProcessDpiAwareness) (HANDLE handle, GdkWin32ProcessDpiAwareness *awareness);
 typedef HRESULT (WINAPI *funcGetDpiForMonitor)       (HMONITOR                monitor,
                                                       GdkWin32MonitorDpiType  dpi_type,
                                                       UINT                   *dpi_x,
@@ -70,53 +69,50 @@ struct _GdkWin32Display
 {
   GdkDisplay display;
 
-  GdkScreen *screen;
+  GdkWin32Screen *screen;
 
   Win32CursorTheme *cursor_theme;
   gchar *cursor_theme_name;
   int cursor_theme_size;
-  GHashTable *cursor_cache;
 
   HWND hwnd;
-  HWND clipboard_hwnd;
 
-  /* OpenGL Items */
-  GdkWin32GLContextType gl_type;
+  /* WGL/OpenGL Items */
+  guint have_wgl : 1;
   guint gl_version;
+  HWND gl_hwnd;
+
+  GListModel *monitors;
 
-  /* WGL Items */
   guint hasWglARBCreateContext : 1;
   guint hasWglEXTSwapControl : 1;
   guint hasWglOMLSyncControl : 1;
   guint hasWglARBPixelFormat : 1;
   guint hasWglARBmultisample : 1;
 
-  /* compensate around Intel OpenGL driver issues on blitting, see issue #3487 */
-  guint needIntelGLWorkaround : 1; 
-
-  /* EGL (Angle) Items */
-  HDC hdc_egl_temp;
-
-#ifdef GDK_WIN32_ENABLE_EGL
-  EGLDisplay egl_disp;
-  EGLConfig egl_config;
-  guint hasEglKHRCreateContext : 1;
-  guint hasEglSurfacelessContext : 1;
-  EGLint egl_min_swap_interval;
-#endif
-
-  GPtrArray *monitors;
-
   /* HiDPI Items */
   guint have_at_least_win81 : 1;
   GdkWin32ProcessDpiAwareness dpi_aware_type;
   guint has_fixed_scale : 1;
-  guint window_scale;
+  guint surface_scale;
 
   GdkWin32ShcoreFuncs shcore_funcs;
   GdkWin32User32DPIFuncs user32_dpi_funcs;
+  
+  /* Cursor Items (GdkCursor->GdkWin32HCursor) */
+  GHashTable *cursors;
+  /* The cursor that is used by current grab (if any) */
+  GdkWin32HCursor *grab_cursor;
+  /* HCURSOR -> GdkWin32HCursorTableEntry */
+  GHashTable *cursor_reftable;
+  /* ID of the idle callback scheduled to destroy cursors */
+  guint idle_cursor_destructor_id;
 
-  /* Running CPU items */
+  /* A list of cursor handles slated for destruction. */
+  GList *cursors_for_destruction;
+
+  /* Message filters */
+  GList *filters;
 };
 
 struct _GdkWin32DisplayClass
@@ -124,12 +120,25 @@ struct _GdkWin32DisplayClass
   GdkDisplayClass display_class;
 };
 
-gboolean   _gdk_win32_display_init_monitors    (GdkWin32Display *display);
+void       _gdk_win32_display_init_monitors    (GdkWin32Display *display);
 
 GPtrArray *_gdk_win32_display_get_monitor_list (GdkWin32Display *display);
 
+void        gdk_win32_display_check_composited (GdkWin32Display *display);
+
 guint      _gdk_win32_display_get_monitor_scale_factor (GdkWin32Display *win32_display,
                                                         HMONITOR         hmonitor,
                                                         HWND             hwnd,
                                                         gint             *dpi);
+
+typedef struct _GdkWin32MessageFilter GdkWin32MessageFilter;
+
+struct _GdkWin32MessageFilter
+{
+  GdkWin32MessageFilterFunc function;
+  gpointer data;
+  gboolean removed;
+  guint ref_count;
+};
+
 #endif /* __GDK_DISPLAY__WIN32_H__ */
