@@ -37,10 +37,10 @@ typedef struct {
   GQuark cpu_time;
   GQuark gpu_time;
 } ProfileTimers;
-#endif
 
 static guint texture_pixels_counter;
 static guint fallback_pixels_counter;
+#endif
 
 struct _GskVulkanRenderer
 {
@@ -211,7 +211,7 @@ gsk_vulkan_renderer_render_texture (GskRenderer           *renderer,
 
   gsk_profiler_push_samples (profiler);
 
-  if (gdk_profiler_is_running ())
+  if (GDK_PROFILER_IS_RUNNING)
     {
       gdk_profiler_add_mark (start_time, cpu_time, "render", "");
       gdk_profiler_set_int_counter (texture_pixels_counter,
