@@ -24,7 +24,7 @@
 #include "gtkapplicationwindow.h"
 #include "gtkwindowprivate.h"
 
-#include "gtkactionmuxer.h"
+#include "gtkactionmuxerprivate.h"
 #include "gtkapplicationaccelsprivate.h"
 
 G_BEGIN_DECLS
@@ -117,7 +117,6 @@ typedef struct
   GtkApplicationImpl impl;
 
   GDBusConnection *session;
-  GCancellable    *cancellable;
 
   const gchar     *application_id;
   const gchar     *unique_name;
