@@ -48,10 +48,10 @@
 #include "config.h"
 
 #include "gtkaspectframe.h"
-#include "gtkbin.h"
 
 #include "gtksizerequest.h"
 
+#include "gtkwidgetprivate.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 
@@ -60,8 +60,9 @@ typedef struct _GtkAspectFrameClass GtkAspectFrameClass;
 
 struct _GtkAspectFrame
 {
-  GtkBin parent_instance;
+  GtkWidget parent_instance;
 
+  GtkWidget    *child;
   gboolean      obey_child;
   float         xalign;
   float         yalign;
@@ -70,7 +71,7 @@ struct _GtkAspectFrame
 
 struct _GtkAspectFrameClass
 {
-  GtkBinClass parent_class;
+  GtkWidgetClass parent_class;
 };
 
 enum {
@@ -78,37 +79,57 @@ enum {
   PROP_XALIGN,
   PROP_YALIGN,
   PROP_RATIO,
-  PROP_OBEY_CHILD
+  PROP_OBEY_CHILD,
+  PROP_CHILD
 };
 
+static void gtk_aspect_frame_dispose      (GObject         *object);
 static void gtk_aspect_frame_set_property (GObject         *object,
-					   guint            prop_id,
-					   const GValue    *value,
-					   GParamSpec      *pspec);
+                                           guint            prop_id,
+                                           const GValue    *value,
+                                           GParamSpec      *pspec);
 static void gtk_aspect_frame_get_property (GObject         *object,
-					   guint            prop_id,
-					   GValue          *value,
-					   GParamSpec      *pspec);
+                                           guint            prop_id,
+                                           GValue          *value,
+                                           GParamSpec      *pspec);
 static void gtk_aspect_frame_size_allocate (GtkWidget      *widget,
                                             int             width,
                                             int             height,
                                             int             baseline);
+static void gtk_aspect_frame_measure       (GtkWidget      *widget,
+                                            GtkOrientation  orientation,
+                                            int             for_size,
+                                            int             *minimum,
+                                            int             *natural,
+                                            int             *minimum_baseline,
+                                            int             *natural_baseline);
+
+static void gtk_aspect_frame_compute_expand (GtkWidget     *widget,
+                                             gboolean      *hexpand,
+                                             gboolean      *vexpand);
+static GtkSizeRequestMode
+            gtk_aspect_frame_get_request_mode (GtkWidget *widget);
+
 
 #define MAX_RATIO 10000.0
 #define MIN_RATIO 0.0001
 
-G_DEFINE_TYPE (GtkAspectFrame, gtk_aspect_frame, GTK_TYPE_BIN)
+G_DEFINE_TYPE (GtkAspectFrame, gtk_aspect_frame, GTK_TYPE_WIDGET)
 
 static void
 gtk_aspect_frame_class_init (GtkAspectFrameClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
-  
+
+  gobject_class->dispose = gtk_aspect_frame_dispose;
   gobject_class->set_property = gtk_aspect_frame_set_property;
   gobject_class->get_property = gtk_aspect_frame_get_property;
 
+  widget_class->measure = gtk_aspect_frame_measure;
   widget_class->size_allocate = gtk_aspect_frame_size_allocate;
+  widget_class->compute_expand = gtk_aspect_frame_compute_expand;
+  widget_class->get_request_mode = gtk_aspect_frame_get_request_mode;
 
   g_object_class_install_property (gobject_class,
                                    PROP_XALIGN,
@@ -139,6 +160,14 @@ gtk_aspect_frame_class_init (GtkAspectFrameClass *class)
                                                          TRUE,
                                                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_CHILD,
+                                   g_param_spec_object ("child",
+                                                        P_("Child"),
+                                                        P_("The child widget"),
+                                                        GTK_TYPE_WIDGET,
+                                                        GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
+
   gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (class), I_("aspectframe"));
 }
 
@@ -152,13 +181,23 @@ gtk_aspect_frame_init (GtkAspectFrame *self)
 }
 
 static void
-gtk_aspect_frame_set_property (GObject         *object,
-			       guint            prop_id,
-			       const GValue    *value,
-			       GParamSpec      *pspec)
+gtk_aspect_frame_dispose (GObject *object)
 {
   GtkAspectFrame *self = GTK_ASPECT_FRAME (object);
-  
+
+  g_clear_pointer (&self->child, gtk_widget_unparent);
+
+  G_OBJECT_CLASS (gtk_aspect_frame_parent_class)->dispose (object);
+}
+
+static void
+gtk_aspect_frame_set_property (GObject         *object,
+                               guint            prop_id,
+                               const GValue    *value,
+                               GParamSpec      *pspec)
+{
+  GtkAspectFrame *self = GTK_ASPECT_FRAME (object);
+
   switch (prop_id)
     {
       /* g_object_notify is handled by the _frame_set function */
@@ -174,6 +213,9 @@ gtk_aspect_frame_set_property (GObject         *object,
     case PROP_OBEY_CHILD:
       gtk_aspect_frame_set_obey_child (self, g_value_get_boolean (value));
       break;
+    case PROP_CHILD:
+      gtk_aspect_frame_set_child (self, g_value_get_object (value));
+      break;
     default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -182,12 +224,12 @@ gtk_aspect_frame_set_property (GObject         *object,
 
 static void
 gtk_aspect_frame_get_property (GObject         *object,
-			       guint            prop_id,
-			       GValue          *value,
-			       GParamSpec      *pspec)
+                               guint            prop_id,
+                               GValue          *value,
+                               GParamSpec      *pspec)
 {
   GtkAspectFrame *self = GTK_ASPECT_FRAME (object);
-  
+
   switch (prop_id)
     {
     case PROP_XALIGN:
@@ -202,6 +244,9 @@ gtk_aspect_frame_get_property (GObject         *object,
     case PROP_OBEY_CHILD:
       g_value_set_boolean (value, self->obey_child);
       break;
+    case PROP_CHILD:
+      g_value_set_object (value, gtk_aspect_frame_get_child (self));
+      break;
     default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -226,9 +271,9 @@ gtk_aspect_frame_get_property (GObject         *object,
  */
 GtkWidget *
 gtk_aspect_frame_new (float    xalign,
-		      float    yalign,
-		      float    ratio,
-		      gboolean obey_child)
+                      float    yalign,
+                      float    ratio,
+                      gboolean obey_child)
 {
   GtkAspectFrame *self;
 
@@ -423,48 +468,45 @@ static void
 compute_child_allocation (GtkAspectFrame *self,
                           GtkAllocation  *child_allocation)
 {
-  GtkBin *bin = GTK_BIN (self);
-  GtkWidget *child;
-  gdouble ratio;
+  double ratio;
 
-  child = gtk_bin_get_child (bin);
-  if (child && gtk_widget_get_visible (child))
+  if (self->child && gtk_widget_get_visible (self->child))
     {
       GtkAllocation full_allocation;
-      
-      if (self->obey_child)
-	{
-	  GtkRequisition child_requisition;
 
-          gtk_widget_get_preferred_size (child, &child_requisition, NULL);
-	  if (child_requisition.height != 0)
-	    {
-	      ratio = ((gdouble) child_requisition.width /
-		       child_requisition.height);
-	      if (ratio < MIN_RATIO)
-		ratio = MIN_RATIO;
-	    }
-	  else if (child_requisition.width != 0)
-	    ratio = MAX_RATIO;
-	  else
-	    ratio = 1.0;
-	}
+      if (self->obey_child)
+        {
+          GtkRequisition child_requisition;
+
+          gtk_widget_get_preferred_size (self->child, &child_requisition, NULL);
+          if (child_requisition.height != 0)
+            {
+              ratio = ((double) child_requisition.width /
+                       child_requisition.height);
+              if (ratio < MIN_RATIO)
+                ratio = MIN_RATIO;
+            }
+          else if (child_requisition.width != 0)
+            ratio = MAX_RATIO;
+          else
+            ratio = 1.0;
+        }
       else
-	ratio = self->ratio;
+        ratio = self->ratio;
 
       get_full_allocation (self, &full_allocation);
-      
+
       if (ratio * full_allocation.height > full_allocation.width)
-	{
-	  child_allocation->width = full_allocation.width;
-	  child_allocation->height = full_allocation.width / ratio + 0.5;
-	}
+        {
+          child_allocation->width = full_allocation.width;
+          child_allocation->height = full_allocation.width / ratio + 0.5;
+        }
       else
-	{
-	  child_allocation->width = ratio * full_allocation.height + 0.5;
-	  child_allocation->height = full_allocation.height;
-	}
-      
+        {
+          child_allocation->width = ratio * full_allocation.height + 0.5;
+          child_allocation->height = full_allocation.height;
+        }
+
       child_allocation->x = full_allocation.x + self->xalign * (full_allocation.width - child_allocation->width);
       child_allocation->y = full_allocation.y + self->yalign * (full_allocation.height - child_allocation->height);
     }
@@ -472,6 +514,36 @@ compute_child_allocation (GtkAspectFrame *self,
     get_full_allocation (self, child_allocation);
 }
 
+static void
+gtk_aspect_frame_measure (GtkWidget      *widget,
+                          GtkOrientation  orientation,
+                          int             for_size,
+                          int             *minimum,
+                          int             *natural,
+                          int             *minimum_baseline,
+                          int             *natural_baseline)
+{
+  GtkAspectFrame *self = GTK_ASPECT_FRAME (widget);
+
+  if (self->child && gtk_widget_get_visible (self->child))
+    {
+      int child_min, child_nat;
+
+      gtk_widget_measure (self->child,
+                          orientation, for_size,
+                          &child_min, &child_nat,
+                          NULL, NULL);
+
+      *minimum = child_min;
+      *natural = child_nat;
+    }
+  else
+    {
+      *minimum = 0;
+      *natural = 0;
+    }
+}
+
 static void
 gtk_aspect_frame_size_allocate (GtkWidget *widget,
                                 int        width,
@@ -479,12 +551,85 @@ gtk_aspect_frame_size_allocate (GtkWidget *widget,
                                 int        baseline)
 {
   GtkAspectFrame *self = GTK_ASPECT_FRAME (widget);
-  GtkWidget *child;
   GtkAllocation new_allocation;
 
   compute_child_allocation (self, &new_allocation);
 
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child && gtk_widget_get_visible (child))
-    gtk_widget_size_allocate (child, &new_allocation, -1);
+  if (self->child && gtk_widget_get_visible (self->child))
+    gtk_widget_size_allocate (self->child, &new_allocation, -1);
 }
+
+static void
+gtk_aspect_frame_compute_expand (GtkWidget *widget,
+                                 gboolean  *hexpand,
+                                 gboolean  *vexpand)
+{
+  GtkAspectFrame *self = GTK_ASPECT_FRAME (widget);
+
+  if (self->child)
+    {
+      *hexpand = gtk_widget_compute_expand (self->child, GTK_ORIENTATION_HORIZONTAL);
+      *vexpand = gtk_widget_compute_expand (self->child, GTK_ORIENTATION_VERTICAL);
+    }
+  else
+    {
+      *hexpand = FALSE;
+      *vexpand = FALSE;
+    }
+}
+
+static GtkSizeRequestMode
+gtk_aspect_frame_get_request_mode (GtkWidget *widget)
+{
+  GtkAspectFrame *self = GTK_ASPECT_FRAME (widget);
+
+  if (self->child)
+    return gtk_widget_get_request_mode (self->child);
+  else
+    return GTK_SIZE_REQUEST_CONSTANT_SIZE;
+}
+
+/**
+ * gtk_aspect_frame_set_child:
+ * @self: a #GtkAspectFrame
+ * @child: (allow-none): the child widget
+ *
+ * Sets the child widget of @self.
+ */
+void
+gtk_aspect_frame_set_child (GtkAspectFrame  *self,
+                            GtkWidget       *child)
+{
+  g_return_if_fail (GTK_IS_ASPECT_FRAME (self));
+  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+
+  if (self->child == child)
+    return;
+
+  g_clear_pointer (&self->child, gtk_widget_unparent);
+
+  if (child)
+    {
+      self->child = child;
+      gtk_widget_set_parent (child, GTK_WIDGET (self));
+    }
+
+  g_object_notify (G_OBJECT (self), "child");
+}
+
+/**
+ * gtk_aspect_frame_get_child:
+ * @self: a #GtkAspectFrame
+ *
+ * Gets the child widget of @self.
+ *
+ * Returns: (nullable) (transfer none): the child widget of self@
+ */
+GtkWidget *
+gtk_aspect_frame_get_child (GtkAspectFrame *self)
+{
+  g_return_val_if_fail (GTK_IS_ASPECT_FRAME (self), NULL);
+
+  return self->child;
+}
+
