@@ -609,12 +609,12 @@ typedef struct {
 } AttributesSubParserData;
 
 static void
-attributes_start_element (GMarkupParseContext *context,
-			  const gchar         *element_name,
-			  const gchar        **names,
-			  const gchar        **values,
-			  gpointer             user_data,
-			  GError             **error)
+attributes_start_element (GtkBuildableParseContext *context,
+                          const gchar              *element_name,
+                          const gchar             **names,
+                          const gchar             **values,
+                          gpointer                  user_data,
+                          GError                  **error)
 {
   AttributesSubParserData *data = (AttributesSubParserData*)user_data;
 
@@ -654,11 +654,11 @@ attributes_start_element (GMarkupParseContext *context,
 }
 
 static void
-attributes_text_element (GMarkupParseContext  *context,
-			 const gchar          *text,
-			 gsize                 text_len,
-			 gpointer              user_data,
-			 GError              **error)
+attributes_text_element (GtkBuildableParseContext  *context,
+                         const gchar               *text,
+                         gsize                      text_len,
+                         gpointer                   user_data,
+                         GError                   **error)
 {
   AttributesSubParserData *data = (AttributesSubParserData*)user_data;
 
@@ -667,10 +667,10 @@ attributes_text_element (GMarkupParseContext  *context,
 }
 
 static void
-attributes_end_element (GMarkupParseContext  *context,
-			const gchar          *element_name,
-			gpointer              user_data,
-			GError              **error)
+attributes_end_element (GtkBuildableParseContext  *context,
+                        const gchar               *element_name,
+                        gpointer                   user_data,
+                        GError                   **error)
 {
   AttributesSubParserData *data = (AttributesSubParserData*)user_data;
   GValue val = G_VALUE_INIT;
@@ -695,7 +695,7 @@ attributes_end_element (GMarkupParseContext  *context,
   g_string_set_size (data->string, 0);
 }
 
-static const GMarkupParser attributes_parser =
+static const GtkBuildableParser attributes_parser =
   {
     attributes_start_element,
     attributes_end_element,
@@ -750,12 +750,12 @@ typedef struct {
 } CellPackingSubParserData;
 
 static void
-cell_packing_start_element (GMarkupParseContext *context,
-			    const gchar         *element_name,
-			    const gchar        **names,
-			    const gchar        **values,
-			    gpointer             user_data,
-			    GError             **error)
+cell_packing_start_element (GtkBuildableParseContext *context,
+                            const gchar              *element_name,
+                            const gchar             **names,
+                            const gchar             **values,
+                            gpointer                  user_data,
+                            GError                  **error)
 {
   CellPackingSubParserData *data = (CellPackingSubParserData*)user_data;
 
@@ -802,11 +802,11 @@ cell_packing_start_element (GMarkupParseContext *context,
 }
 
 static void
-cell_packing_text_element (GMarkupParseContext *context,
-			   const gchar         *text,
-			   gsize                text_len,
-			   gpointer             user_data,
-			   GError             **error)
+cell_packing_text_element (GtkBuildableParseContext *context,
+                           const gchar              *text,
+                           gsize                     text_len,
+                           gpointer                  user_data,
+                           GError                  **error)
 {
   CellPackingSubParserData *data = (CellPackingSubParserData*)user_data;
 
@@ -815,10 +815,10 @@ cell_packing_text_element (GMarkupParseContext *context,
 }
 
 static void
-cell_packing_end_element (GMarkupParseContext *context,
-			  const gchar         *element_name,
-			  gpointer             user_data,
-			  GError             **error)
+cell_packing_end_element (GtkBuildableParseContext *context,
+                          const gchar              *element_name,
+                          gpointer                  user_data,
+                          GError                  **error)
 {
   CellPackingSubParserData *data = (CellPackingSubParserData*)user_data;
   GtkCellArea *area;
@@ -861,7 +861,7 @@ cell_packing_end_element (GMarkupParseContext *context,
 }
 
 
-static const GMarkupParser cell_packing_parser =
+static const GtkBuildableParser cell_packing_parser =
   {
     cell_packing_start_element,
     cell_packing_end_element,
@@ -869,12 +869,12 @@ static const GMarkupParser cell_packing_parser =
   };
 
 gboolean
-_gtk_cell_layout_buildable_custom_tag_start (GtkBuildable  *buildable,
-					     GtkBuilder    *builder,
-					     GObject       *child,
-					     const gchar   *tagname,
-					     GMarkupParser *parser,
-					     gpointer      *data)
+_gtk_cell_layout_buildable_custom_tag_start (GtkBuildable       *buildable,
+                                             GtkBuilder         *builder,
+                                             GObject            *child,
+                                             const gchar        *tagname,
+                                             GtkBuildableParser *parser,
+                                             gpointer           *data)
 {
   AttributesSubParserData  *attr_data;
   CellPackingSubParserData *packing_data;
