@@ -19,7 +19,6 @@
 #include <string.h>
 
 #include "gtkintl.h"
-#include "gtkbin.h"
 #include "gtkentry.h"
 #include "gtkcelllayout.h"
 #include "gtkcellrenderercombo.h"
@@ -373,7 +372,7 @@ gtk_cell_renderer_combo_editing_done (GtkCellEditable *combo,
 
   if (gtk_combo_box_get_has_entry (GTK_COMBO_BOX (combo)))
     {
-      entry = GTK_ENTRY (gtk_bin_get_child (GTK_BIN (combo)));
+      entry = GTK_ENTRY (gtk_combo_box_get_child (GTK_COMBO_BOX (combo)));
       new_text = g_strdup (gtk_editable_get_text (GTK_EDITABLE (entry)));
     }
   else 
@@ -469,7 +468,7 @@ gtk_cell_renderer_combo_start_editing (GtkCellRenderer     *cell,
 
       g_object_get (cell, "text", &text, NULL);
       if (text)
-	gtk_editable_set_text (GTK_EDITABLE (gtk_bin_get_child (GTK_BIN (combo))), text);
+	gtk_editable_set_text (GTK_EDITABLE (gtk_combo_box_get_child (GTK_COMBO_BOX (combo))), text);
       g_free (text);
     }
   else
