@@ -600,15 +600,16 @@ gtk_entry_completion_constructed (GObject *object)
   gtk_widget_add_controller (priv->popup_window, controller);
 
   popup_frame = gtk_frame_new (NULL);
-  gtk_container_add (GTK_CONTAINER (priv->popup_window), popup_frame);
+  gtk_popover_set_child (GTK_POPOVER (priv->popup_window), popup_frame);
 
   priv->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-  gtk_container_add (GTK_CONTAINER (popup_frame), priv->vbox);
+  gtk_frame_set_child (GTK_FRAME (popup_frame), priv->vbox);
 
-  gtk_container_add (GTK_CONTAINER (priv->scrolled_window), priv->tree_view);
+  gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (priv->scrolled_window),
+                                 priv->tree_view);
   gtk_widget_set_hexpand (priv->scrolled_window, TRUE);
   gtk_widget_set_vexpand (priv->scrolled_window, TRUE);
-  gtk_container_add (GTK_CONTAINER (priv->vbox), priv->scrolled_window);
+  gtk_box_append (GTK_BOX (priv->vbox), priv->scrolled_window);
 
   /* we don't want to see the action treeview when no actions have
    * been inserted, so we pack the action treeview after the first
@@ -1196,7 +1197,7 @@ gtk_entry_completion_insert_action (GtkEntryCompletion *completion,
                                 path, NULL, FALSE);
       gtk_tree_path_free (path);
 
-      gtk_container_add (GTK_CONTAINER (completion->priv->vbox),
+      gtk_box_append (GTK_BOX (completion->priv->vbox),
                           completion->priv->action_view);
       gtk_widget_show (completion->priv->action_view);
     }
