@@ -2250,7 +2250,8 @@ update_label_and_image (GtkFileChooserButton *button)
 	    }
         }
 
-      if (g_file_is_native (file))
+      if (g_file_is_native (file) ||
+          !_gtk_bookmarks_manager_has_bookmark (button->bookmarks_manager, file))
         {
           button->update_button_cancellable =
             _gtk_file_system_get_info (button->fs, file,
