@@ -20,7 +20,7 @@
 
 #include "config.h"
 
-#include "gtkoverlaylayoutprivate.h"
+#include "gtkoverlaylayout.h"
 
 #include "gtkintl.h"
 #include "gtklayoutchild.h"
@@ -121,12 +121,23 @@ gtk_overlay_layout_child_class_init (GtkOverlayLayoutChildClass *klass)
   gobject_class->get_property = gtk_overlay_layout_child_get_property;
   gobject_class->finalize = gtk_overlay_layout_child_finalize;
 
+  /**
+   * GtkOverlayLayoutChild:measure:
+   *
+   * Whether the child size should contribute to the #GtkOverlayLayout's
+   * measurement.
+   */
   child_props[PROP_MEASURE] =
     g_param_spec_boolean ("measure",
                           P_("Measure"),
                           P_("Include in size measurement"),
                           FALSE,
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
+  /**
+   * GtkOverlayLayoutChild:clip-overlay:
+   *
+   * Whether the child should be clipped to fit the parent's size.
+   */
   child_props[PROP_CLIP_OVERLAY] =
     g_param_spec_boolean ("clip-overlay",
                           P_("Clip Overlay"),
@@ -174,7 +185,7 @@ gtk_overlay_layout_child_set_measure (GtkOverlayLayoutChild *child,
  *
  * Retrieves whether the child is measured.
  *
- * Returns: (transfer none) (nullable): whether the child is measured
+ * Returns: whether the child is measured
  */
 gboolean
 gtk_overlay_layout_child_get_measure (GtkOverlayLayoutChild *child)
@@ -216,7 +227,7 @@ gtk_overlay_layout_child_set_clip_overlay (GtkOverlayLayoutChild *child,
  *
  * Retrieves whether the child is clipped.
  *
- * Returns: (transfer none) (nullable): whether the child is clipped
+ * Returns: whether the child is clipped
  */
 gboolean
 gtk_overlay_layout_child_get_clip_overlay (GtkOverlayLayoutChild *child)
@@ -443,6 +454,13 @@ gtk_overlay_layout_init (GtkOverlayLayout *self)
 {
 }
 
+/**
+ * gtk_overlay_layout_new:
+ *
+ * Creates a new #GtkOverlayLayout instance.
+ *
+ * Returns: the newly created instance
+ */
 GtkLayoutManager *
 gtk_overlay_layout_new (void)
 {
