@@ -26,6 +26,7 @@
 
 #include "gtkpaned.h"
 
+#include "gtkcssboxesprivate.h"
 #include "gtkcssnodeprivate.h"
 #include "gtkcssstylepropertyprivate.h"
 #include "gtkeventcontrollermotion.h"
@@ -34,7 +35,7 @@
 #include "gtkgizmoprivate.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
-#include "gtkorientableprivate.h"
+#include "gtkorientable.h"
 #include "gtkprivate.h"
 #include "gtkrendericonprivate.h"
 #include "gtkstylecontextprivate.h"
@@ -108,10 +109,14 @@
  *
  * gtk_widget_set_size_request (hpaned, 200, -1);
  *
- * gtk_paned_pack1 (GTK_PANED (hpaned), frame1, TRUE, FALSE);
+ * gtk_paned_set_start_child (GTK_PANED (hpaned), frame1);
+ * gtk_paned_set_start_child_resize (GTK_PANED (hpaned), TRUE);
+ * gtk_paned_set_start_child_shrink (GTK_PANED (hpaned), FALSE);
  * gtk_widget_set_size_request (frame1, 50, -1);
  *
- * gtk_paned_pack2 (GTK_PANED (hpaned), frame2, FALSE, FALSE);
+ * gtk_paned_set_end_child (GTK_PANED (hpaned), frame2);
+ * gtk_paned_set_end_child_resize (GTK_PANED (hpaned), FALSE);
+ * gtk_paned_set_end_child_shrink (GTK_PANED (hpaned), FALSE);
  * gtk_widget_set_size_request (frame2, 50, -1);
  * ]|
  */
@@ -389,6 +394,30 @@ gtk_paned_get_request_mode (GtkWidget *widget)
         GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
 }
 
+static void
+gtk_paned_set_orientation (GtkPaned       *self,
+                           GtkOrientation  orientation)
+{
+  if (self->orientation != orientation)
+    {
+      static const char *cursor_name[2] = {
+        "col-resize",
+        "row-resize",
+      };
+
+      self->orientation = orientation;
+
+      gtk_widget_update_orientation (GTK_WIDGET (self), self->orientation);
+      gtk_widget_set_cursor_from_name (self->handle_widget,
+                                       cursor_name[orientation]);
+      gtk_gesture_pan_set_orientation (GTK_GESTURE_PAN (self->pan_gesture),
+                                       orientation);
+
+      gtk_widget_queue_resize (GTK_WIDGET (self));
+      g_object_notify (G_OBJECT (self), "orientation");
+    }
+}
+
 static void
 gtk_paned_class_init (GtkPanedClass *class)
 {
@@ -919,29 +948,7 @@ gtk_paned_set_property (GObject        *object,
   switch (prop_id)
     {
     case PROP_ORIENTATION:
-      if (paned->orientation != g_value_get_enum (value))
-        {
-          paned->orientation = g_value_get_enum (value);
-          _gtk_orientable_set_style_classes (GTK_ORIENTABLE (paned));
-
-          if (paned->orientation == GTK_ORIENTATION_HORIZONTAL)
-            {
-              gtk_gesture_pan_set_orientation (GTK_GESTURE_PAN (paned->pan_gesture),
-                                               GTK_ORIENTATION_HORIZONTAL);
-              gtk_widget_set_cursor_from_name (paned->handle_widget,
-                                               "col-resize");
-            }
-          else
-            {
-              gtk_gesture_pan_set_orientation (GTK_GESTURE_PAN (paned->pan_gesture),
-                                               GTK_ORIENTATION_VERTICAL);
-              gtk_widget_set_cursor_from_name (paned->handle_widget,
-                                               "row-resize");
-            }
-
-          gtk_widget_queue_resize (GTK_WIDGET (paned));
-          g_object_notify_by_pspec (object, pspec);
-        }
+      gtk_paned_set_orientation (paned, g_value_get_enum (value));
       break;
     case PROP_POSITION:
       gtk_paned_set_position (paned, g_value_get_int (value));
@@ -1450,7 +1457,7 @@ gtk_paned_init (GtkPaned *paned)
   paned->shrink_start_child = TRUE;
   paned->shrink_end_child = TRUE;
 
-  _gtk_orientable_set_style_classes (GTK_ORIENTABLE (paned));
+  gtk_widget_update_orientation (GTK_WIDGET (paned), paned->orientation);
 
   /* Touch gesture */
   gesture = gtk_gesture_pan_new (GTK_ORIENTATION_HORIZONTAL);
@@ -1552,6 +1559,13 @@ gtk_paned_new (GtkOrientation orientation)
                        NULL);
 }
 
+/**
+ * gtk_paned_set_start_child:
+ * @paned: a #GtkPaned
+ * @child: the widget to add
+ *
+ * Sets the start child of @paned to @child.
+ */
 void
 gtk_paned_set_start_child (GtkPaned *paned,
                            GtkWidget *child)
@@ -1588,6 +1602,13 @@ gtk_paned_get_start_child (GtkPaned *paned)
   return paned->start_child;
 }
 
+/**
+ * gtk_paned_set_resize_start_child:
+ * @paned: a #GtkPaned
+ * @resize: %TRUE to let the start child be resized
+ *
+ * Sets the #GtkPaned:resize-start-child property
+ */
 void
 gtk_paned_set_resize_start_child (GtkPaned *paned,
                                   gboolean  resize)
@@ -1602,6 +1623,14 @@ gtk_paned_set_resize_start_child (GtkPaned *paned,
   g_object_notify (G_OBJECT (paned), "resize-start-child");
 }
 
+/**
+ * gtk_paned_get_resize_start_child:
+ * @paned: a #GtkPaned
+ *
+ * Returns whether the start child can be resized.
+ *
+ * Returns: %TRUE if the start child is resizable
+ */
 gboolean
 gtk_paned_get_resize_start_child (GtkPaned *paned)
 {
@@ -1610,6 +1639,13 @@ gtk_paned_get_resize_start_child (GtkPaned *paned)
   return paned->resize_start_child;
 }
 
+/**
+ * gtk_paned_set_shrink_start_child:
+ * @paned: a #GtkPaned
+ * @resize: %TRUE to let the start child be shrunk
+ *
+ * Sets the #GtkPaned:shrink-start-child property
+ */
 void
 gtk_paned_set_shrink_start_child (GtkPaned *paned,
                                   gboolean  shrink)
@@ -1624,6 +1660,14 @@ gtk_paned_set_shrink_start_child (GtkPaned *paned,
   g_object_notify (G_OBJECT (paned), "shrink-start-child");
 }
 
+/**
+ * gtk_paned_get_shrink_start_child:
+ * @paned: a #GtkPaned
+ *
+ * Returns whether the start child can be shrunk.
+ *
+ * Returns: %TRUE if the start child is shrinkable
+ */
 gboolean
 gtk_paned_get_shrink_start_child (GtkPaned *paned)
 {
@@ -1633,14 +1677,11 @@ gtk_paned_get_shrink_start_child (GtkPaned *paned)
 }
 
 /**
- * gtk_paned_get_end_child:
+ * gtk_paned_set_end_child:
  * @paned: a #GtkPaned
+ * @child: the widget to add
  *
- * Retrieves the end child of the given #GtkPaned.
- *
- * See also: #GtkPaned:end-child
- *
- * Returns: (transfer none) (nullable): the end child widget
+ * Sets the end child of @paned to @child.
  */
 void
 gtk_paned_set_end_child (GtkPaned *paned,
@@ -1660,6 +1701,16 @@ gtk_paned_set_end_child (GtkPaned *paned,
   g_object_notify (G_OBJECT (paned), "end-child");
 }
 
+/**
+ * gtk_paned_get_end_child:
+ * @paned: a #GtkPaned
+ *
+ * Retrieves the end child of the given #GtkPaned.
+ *
+ * See also: #GtkPaned:end-child
+ *
+ * Returns: (transfer none) (nullable): the end child widget
+ */
 GtkWidget *
 gtk_paned_get_end_child (GtkPaned *paned)
 {
@@ -1668,6 +1719,13 @@ gtk_paned_get_end_child (GtkPaned *paned)
   return paned->end_child;
 }
 
+/**
+ * gtk_paned_set_resize_end_child:
+ * @paned: a #GtkPaned
+ * @resize: %TRUE to let the end child be resized
+ *
+ * Sets the #GtkPaned:resize-end-child property
+ */
 void
 gtk_paned_set_resize_end_child (GtkPaned *paned,
                                 gboolean  resize)
@@ -1682,6 +1740,14 @@ gtk_paned_set_resize_end_child (GtkPaned *paned,
   g_object_notify (G_OBJECT (paned), "resize-end-child");
 }
 
+/**
+ * gtk_paned_get_resize_end_child:
+ * @paned: a #GtkPaned
+ *
+ * Returns whether the end child can be resized.
+ *
+ * Returns: %TRUE if the end child is resizable
+ */
 gboolean
 gtk_paned_get_resize_end_child (GtkPaned *paned)
 {
@@ -1690,6 +1756,13 @@ gtk_paned_get_resize_end_child (GtkPaned *paned)
   return paned->resize_end_child;
 }
 
+/**
+ * gtk_paned_set_shrink_end_child:
+ * @paned: a #GtkPaned
+ * @resize: %TRUE to let the end child be shrunk
+ *
+ * Sets the #GtkPaned:shrink-end-child property
+ */
 void
 gtk_paned_set_shrink_end_child (GtkPaned *paned,
                                 gboolean  shrink)
@@ -1704,6 +1777,14 @@ gtk_paned_set_shrink_end_child (GtkPaned *paned,
   g_object_notify (G_OBJECT (paned), "shrink-end-child");
 }
 
+/**
+ * gtk_paned_get_shrink_end_child:
+ * @paned: a #GtkPaned
+ *
+ * Returns whether the end child can be shrunk.
+ *
+ * Returns: %TRUE if the end child is shrinkable
+ */
 gboolean
 gtk_paned_get_shrink_end_child (GtkPaned *paned)
 {
