@@ -20,7 +20,7 @@
 
 #include "config.h"
 
-#include "gtkpasswordentry.h"
+#include "gtkpasswordentryprivate.h"
 
 #include "gtkaccessible.h"
 #include "gtktextprivate.h"
@@ -35,7 +35,7 @@
 #include "gtkstylecontext.h"
 #include "gtkeventcontrollerkey.h"
 
-#include "a11y/gtkentryaccessible.h"
+#include "a11y/gtkpasswordentryaccessibleprivate.h"
 
 /**
  * SECTION:gtkpasswordentry
@@ -116,6 +116,11 @@ gtk_password_entry_toggle_peek (GtkPasswordEntry *entry)
 
   visibility = gtk_text_get_visibility (GTK_TEXT (priv->entry));
   gtk_text_set_visibility (GTK_TEXT (priv->entry), !visibility);
+
+  /* Update the accessible object to reflect the change of visibility */
+  GtkAccessible *accessible = GTK_ACCESSIBLE (_gtk_widget_peek_accessible (GTK_WIDGET (entry)));
+  if (accessible != NULL)
+    gtk_password_entry_accessible_update_visibility (GTK_PASSWORD_ENTRY_ACCESSIBLE (accessible));
 }
 
 static void
@@ -346,17 +351,6 @@ gtk_password_entry_size_allocate (GtkWidget *widget,
                               baseline);
 }
 
-static AtkObject *
-gtk_password_entry_get_accessible (GtkWidget *widget)
-{
-  AtkObject *atk_obj;
-
-  atk_obj = GTK_WIDGET_CLASS (gtk_password_entry_parent_class)->get_accessible (widget);
-  atk_object_set_name (atk_obj, _("Password"));
-
-  return atk_obj;
-}
-
 static gboolean
 gtk_password_entry_mnemonic_activate (GtkWidget *widget,
                                       gboolean   group_cycling)
@@ -382,7 +376,6 @@ gtk_password_entry_class_init (GtkPasswordEntryClass *klass)
   widget_class->realize = gtk_password_entry_realize;
   widget_class->measure = gtk_password_entry_measure;
   widget_class->size_allocate = gtk_password_entry_size_allocate;
-  widget_class->get_accessible = gtk_password_entry_get_accessible;
   widget_class->mnemonic_activate = gtk_password_entry_mnemonic_activate;
   widget_class->grab_focus = gtk_widget_grab_focus_child;
 
@@ -423,7 +416,7 @@ gtk_password_entry_class_init (GtkPasswordEntryClass *klass)
   g_object_class_install_properties (object_class, NUM_PROPERTIES, props);
   gtk_editable_install_properties (object_class, NUM_PROPERTIES);
 
-  gtk_widget_class_set_accessible_type (widget_class, GTK_TYPE_ENTRY_ACCESSIBLE);
+  gtk_widget_class_set_accessible_type (widget_class, GTK_TYPE_PASSWORD_ENTRY_ACCESSIBLE);
   gtk_widget_class_set_css_name (widget_class, I_("entry"));
 }
 
@@ -442,6 +435,16 @@ gtk_password_entry_editable_init (GtkEditableInterface *iface)
   iface->get_delegate = gtk_password_entry_get_delegate;
 }
 
+GtkText *
+gtk_password_entry_get_text_widget (GtkPasswordEntry *entry)
+{
+  GtkPasswordEntryPrivate *priv = gtk_password_entry_get_instance_private (entry);
+
+  g_return_val_if_fail (GTK_IS_PASSWORD_ENTRY (entry), NULL);
+
+  return GTK_TEXT (priv->entry);
+}
+
 /**
  * gtk_password_entry_new:
  *
