@@ -39,11 +39,6 @@
 
 #include "gsk/gskroundedrectprivate.h"
 
-/* this is in case round() is not provided by the compiler, 
- * such as in the case of C89 compilers, like MSVC
- */
-#include "fallback-c89.c"
-
 typedef struct _GtkBorderImage GtkBorderImage;
 
 struct _GtkBorderImage {
