@@ -119,8 +119,6 @@ typedef struct
   GtkScrollType autoscroll_step;
   gboolean autoscrolling;
 
-  guint click_id;
-
   gchar **icon_list;
 
   GtkAdjustment *adjustment; /* needed because it must be settable in init() */
@@ -409,7 +407,6 @@ gtk_scale_button_init (GtkScaleButton *button)
   GtkScaleButtonPrivate *priv = gtk_scale_button_get_instance_private (button);
   GtkEventController *controller;
 
-  priv->click_id = 0;
   priv->orientation = GTK_ORIENTATION_VERTICAL;
   priv->applied_orientation = GTK_ORIENTATION_VERTICAL;
 
@@ -515,20 +512,9 @@ gtk_scale_button_finalize (GObject *object)
   GtkScaleButton *button = GTK_SCALE_BUTTON (object);
   GtkScaleButtonPrivate *priv = gtk_scale_button_get_instance_private (button);
 
-  if (priv->icon_list)
-    {
-      g_strfreev (priv->icon_list);
-      priv->icon_list = NULL;
-    }
-
-  if (priv->adjustment)
-    {
-      g_object_unref (priv->adjustment);
-      priv->adjustment = NULL;
-    }
-
-  if (priv->autoscroll_timeout)
-    g_source_remove (priv->autoscroll_timeout);
+  g_clear_pointer (&priv->icon_list, g_strfreev);
+  g_clear_object (&priv->adjustment);
+  g_clear_handle_id (&priv->autoscroll_timeout, g_source_remove);
 
   G_OBJECT_CLASS (gtk_scale_button_parent_class)->finalize (object);
 }
@@ -542,12 +528,6 @@ gtk_scale_button_dispose (GObject *object)
   g_clear_pointer (&priv->dock, gtk_widget_unparent);
   g_clear_pointer (&priv->button, gtk_widget_unparent);
 
-  if (priv->click_id != 0)
-    {
-      g_source_remove (priv->click_id);
-      priv->click_id = 0;
-    }
-
   G_OBJECT_CLASS (gtk_scale_button_parent_class)->dispose (object);
 }
 
