@@ -21,20 +21,20 @@
 
 #include "config.h"
 #include "gtksearchengine.h"
-#include "gtksearchenginetracker.h"
 #include "gtksearchenginemodel.h"
 #include "gtksearchenginequartz.h"
 #include "gtkintl.h"
 
 #include <gdk/gdk.h> /* for GDK_WINDOWING_QUARTZ */
 
-#ifndef G_OS_WIN32  /* No tracker on Windows */
-#define HAVE_TRACKER 1
+#if defined(HAVE_TRACKER3)
+#include "gtksearchenginetracker3.h"
 #endif
 
 struct _GtkSearchEnginePrivate {
   GtkSearchEngine *native;
   gboolean native_running;
+  gboolean got_results;
   gchar *native_error;
 
   GtkSearchEngine *model;
@@ -161,7 +161,7 @@ _gtk_search_engine_class_init (GtkSearchEngineClass *class)
                   G_STRUCT_OFFSET (GtkSearchEngineClass, finished),
                   NULL, NULL,
                   NULL,
-                  G_TYPE_NONE, 0);
+                  G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 
   signals[ERROR] =
     g_signal_new (I_("error"),
@@ -230,13 +230,16 @@ update_status (GtkSearchEngine *engine)
           else if (engine->priv->model_error)
             _gtk_search_engine_error (engine, engine->priv->model_error);
           else
-            _gtk_search_engine_finished (engine);
+            _gtk_search_engine_finished (engine, engine->priv->got_results);
+
+          engine->priv->got_results = FALSE;
         }
     }
 }
 
 static void
 finished (GtkSearchEngine *engine,
+          gboolean         got_results,
           gpointer         data)
 {
   GtkSearchEngine *composite = GTK_SEARCH_ENGINE (data);
@@ -246,6 +249,7 @@ finished (GtkSearchEngine *engine,
   else if (engine == composite->priv->model)
     composite->priv->model_running = FALSE;
 
+  composite->priv->got_results |= got_results;
   update_status (composite);
 }
 
@@ -329,11 +333,11 @@ _gtk_search_engine_new (void)
 
   engine = g_object_new (GTK_TYPE_SEARCH_ENGINE, NULL);
 
-#ifdef HAVE_TRACKER
-  engine->priv->native = _gtk_search_engine_tracker_new ();
+#if defined(HAVE_TRACKER3)
+  engine->priv->native = gtk_search_engine_tracker3_new ();
   if (engine->priv->native)
     {
-      g_debug ("Using Tracker search engine");
+      g_debug ("Using Tracker3 search engine");
       connect_engine_signals (engine->priv->native, engine);
     }
 #endif
@@ -391,11 +395,12 @@ _gtk_search_engine_hits_added (GtkSearchEngine *engine,
 }
 
 void
-_gtk_search_engine_finished (GtkSearchEngine *engine)
+_gtk_search_engine_finished (GtkSearchEngine *engine,
+                             gboolean         got_results)
 {
   g_return_if_fail (GTK_IS_SEARCH_ENGINE (engine));
 
-  g_signal_emit (engine, signals[FINISHED], 0);
+  g_signal_emit (engine, signals[FINISHED], 0, got_results);
 }
 
 void
