@@ -56,6 +56,7 @@
 #include "gtkbuilder.h"
 #include "gtkintl.h"
 #include "gtkwidgetprivate.h"
+#include "gtkdebug.h"
 
 /* {{{ GtkShortcutAction */
 
@@ -158,6 +159,12 @@ gtk_shortcut_action_activate (GtkShortcutAction      *self,
   g_return_val_if_fail (GTK_IS_SHORTCUT_ACTION (self), FALSE);
   g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
+  GTK_NOTE (SHORTCUTS, {
+            char *act = gtk_shortcut_action_to_string (self);
+            g_print ("Shortcut action activate on %s: %s\n", G_OBJECT_TYPE_NAME (widget), act);
+            g_free (act);
+            });
+
   return GTK_SHORTCUT_ACTION_GET_CLASS (self)->activate (self, flags, widget, args);
 }
 
