@@ -28,10 +28,10 @@
  * SECTION:gtksortlistmodel
  * @title: GtkSortListModel
  * @short_description: A list model that sorts its items
- * @see_also: #GListModel
+ * @see_also: #GListModel, #GtkSorter
  *
  * #GtkSortListModel is a list model that takes a list model and
- * sorts its elements according to a compare function.
+ * sorts its elements according to a #GtkSorter.
  *
  * #GtkSortListModel is a generic model and because of that it
  * cannot take advantage of any external knowledge when sorting.
@@ -42,24 +42,24 @@
 
 enum {
   PROP_0,
-  PROP_HAS_SORT,
   PROP_ITEM_TYPE,
   PROP_MODEL,
+  PROP_SORTER,
   NUM_PROPERTIES
 };
 
+typedef struct _GtkSortListEntry GtkSortListEntry;
+
 struct _GtkSortListModel
 {
   GObject parent_instance;
 
   GType item_type;
   GListModel *model;
-  GCompareDataFunc sort_func;
-  gpointer user_data;
-  GDestroyNotify user_destroy;
+  GtkSorter *sorter;
 
-  GSequence *sorted; /* NULL if sort_func == NULL */
-  GSequence *unsorted; /* NULL if sort_func == NULL */
+  GSequence *sorted; /* NULL if known unsorted */
+  GSequence *unsorted; /* NULL if known unsorted */
 };
 
 struct _GtkSortListModelClass
@@ -67,8 +67,25 @@ struct _GtkSortListModelClass
   GObjectClass parent_class;
 };
 
+struct _GtkSortListEntry
+{
+  GSequenceIter *sorted_iter;
+  GSequenceIter *unsorted_iter;
+  gpointer item; /* holds ref */
+};
+
 static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 
+static void
+gtk_sort_list_entry_free (gpointer data)
+{
+  GtkSortListEntry *entry = data;
+
+  g_object_unref (entry->item);
+
+  g_slice_free (GtkSortListEntry, entry);
+}
+
 static GType
 gtk_sort_list_model_get_item_type (GListModel *list)
 {
@@ -85,9 +102,6 @@ gtk_sort_list_model_get_n_items (GListModel *list)
   if (self->model == NULL)
     return 0;
 
-  if (self->sorted)
-    return g_sequence_get_length (self->sorted);
-
   return g_list_model_get_n_items (self->model);
 }
 
@@ -97,6 +111,7 @@ gtk_sort_list_model_get_item (GListModel *list,
 {
   GtkSortListModel *self = GTK_SORT_LIST_MODEL (list);
   GSequenceIter *iter;
+  GtkSortListEntry *entry;
 
   if (self->model == NULL)
     return NULL;
@@ -106,9 +121,11 @@ gtk_sort_list_model_get_item (GListModel *list,
 
   iter = g_sequence_get_iter_at_pos (self->sorted, position);
   if (g_sequence_iter_is_end (iter))
-      return NULL;
+    return NULL;
 
-  return g_object_ref (g_sequence_get (iter));
+  entry = g_sequence_get (iter);
+
+  return g_object_ref (entry->item);
 }
 
 static void
@@ -137,18 +154,18 @@ gtk_sort_list_model_remove_items (GtkSortListModel *self,
 
   for (i = 0; i < n_items ; i++)
     {
-      GSequenceIter *sorted_iter;
+      GtkSortListEntry *entry;
       GSequenceIter *next;
 
       next = g_sequence_iter_next (unsorted_iter);
 
-      sorted_iter = g_sequence_get (unsorted_iter);
-      pos = g_sequence_iter_get_position (sorted_iter);
+      entry = g_sequence_get (unsorted_iter);
+      pos = g_sequence_iter_get_position (entry->sorted_iter);
       start = MIN (start, pos);
       end = MIN (end, length_before - i - 1 - pos);
 
-      g_sequence_remove (sorted_iter);
-      g_sequence_remove (unsorted_iter);
+      g_sequence_remove (entry->unsorted_iter);
+      g_sequence_remove (entry->sorted_iter);
 
       unsorted_iter = next;
     }
@@ -157,6 +174,23 @@ gtk_sort_list_model_remove_items (GtkSortListModel *self,
   *unmodified_end = end;
 }
 
+static int
+_sort_func (gconstpointer item1,
+            gconstpointer item2,
+            gpointer      data)
+{
+  GtkSortListEntry *entry1 = (GtkSortListEntry *) item1;
+  GtkSortListEntry *entry2 = (GtkSortListEntry *) item2;
+  GtkOrdering result;
+
+  result = gtk_sorter_compare (GTK_SORTER (data), entry1->item, entry2->item);
+
+  if (result == GTK_ORDERING_EQUAL)
+    result = g_sequence_iter_compare (entry1->unsorted_iter, entry2->unsorted_iter);
+
+  return result;
+}
+
 static void
 gtk_sort_list_model_add_items (GtkSortListModel *self,
                                guint             position,
@@ -164,20 +198,22 @@ gtk_sort_list_model_add_items (GtkSortListModel *self,
                                guint            *unmodified_start,
                                guint            *unmodified_end)
 {
-  GSequenceIter *unsorted_iter, *sorted_iter;
+  GSequenceIter *unsorted_end;
   guint i, pos, start, end, length_before;
 
-  unsorted_iter = g_sequence_get_iter_at_pos (self->unsorted, position);
+  unsorted_end = g_sequence_get_iter_at_pos (self->unsorted, position);
   start = end = length_before = g_sequence_get_length (self->sorted);
 
   for (i = 0; i < n_items; i++)
     {
-      gpointer item = g_list_model_get_item (self->model, position + i);
-      sorted_iter = g_sequence_insert_sorted (self->sorted, item, self->sort_func, self->user_data);
-      g_sequence_insert_before (unsorted_iter, sorted_iter);
+      GtkSortListEntry *entry = g_slice_new0 (GtkSortListEntry);
+
+      entry->item = g_list_model_get_item (self->model, position + i);
+      entry->unsorted_iter = g_sequence_insert_before (unsorted_end, entry);
+      entry->sorted_iter = g_sequence_insert_sorted (self->sorted, entry, _sort_func, self->sorter);
       if (unmodified_start != NULL || unmodified_end != NULL)
         {
-          pos = g_sequence_iter_get_position (sorted_iter);
+          pos = g_sequence_iter_get_position (entry->sorted_iter);
           start = MIN (start, pos);
           end = MIN (end, length_before + i - pos);
         }
@@ -234,6 +270,10 @@ gtk_sort_list_model_set_property (GObject      *object,
       gtk_sort_list_model_set_model (self, g_value_get_object (value));
       break;
 
+    case PROP_SORTER:
+      gtk_sort_list_model_set_sorter (self, g_value_get_object (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -250,10 +290,6 @@ gtk_sort_list_model_get_property (GObject     *object,
 
   switch (prop_id)
     {
-    case PROP_HAS_SORT:
-      g_value_set_boolean (value, self->sort_func != NULL);
-      break;
-
     case PROP_ITEM_TYPE:
       g_value_set_gtype (value, self->item_type);
       break;
@@ -262,12 +298,26 @@ gtk_sort_list_model_get_property (GObject     *object,
       g_value_set_object (value, self->model);
       break;
 
+    case PROP_SORTER:
+      g_value_set_object (value, self->sorter);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
+static void gtk_sort_list_model_resort (GtkSortListModel *self);
+
+static void
+gtk_sort_list_model_sorter_changed_cb (GtkSorter        *sorter,
+                                       int               change,
+                                       GtkSortListModel *self)
+{
+  gtk_sort_list_model_resort (self);
+}
+
 static void
 gtk_sort_list_model_clear_model (GtkSortListModel *self)
 {
@@ -280,17 +330,23 @@ gtk_sort_list_model_clear_model (GtkSortListModel *self)
   g_clear_pointer (&self->unsorted, g_sequence_free);
 }
 
+static void
+gtk_sort_list_model_clear_sorter (GtkSortListModel *self)
+{
+  if (self->sorter == NULL)
+    return;
+
+  g_signal_handlers_disconnect_by_func (self->sorter, gtk_sort_list_model_sorter_changed_cb, self);
+  g_clear_object (&self->sorter);
+}
+
 static void
 gtk_sort_list_model_dispose (GObject *object)
 {
   GtkSortListModel *self = GTK_SORT_LIST_MODEL (object);
 
   gtk_sort_list_model_clear_model (self);
-  if (self->user_destroy)
-    self->user_destroy (self->user_data);
-  self->sort_func = NULL;
-  self->user_data = NULL;
-  self->user_destroy = NULL;
+  gtk_sort_list_model_clear_sorter (self);
 
   G_OBJECT_CLASS (gtk_sort_list_model_parent_class)->dispose (object);
 };
@@ -305,16 +361,16 @@ gtk_sort_list_model_class_init (GtkSortListModelClass *class)
   gobject_class->dispose = gtk_sort_list_model_dispose;
 
   /**
-   * GtkSortListModel:has-sort:
+   * GtkSortListModel:sorter:
    *
-   * If a sort function is set for this model
+   * The sorter for this model
    */
-  properties[PROP_HAS_SORT] =
-      g_param_spec_boolean ("has-sort",
-                            P_("has sort"),
-                            P_("If a sort function is set for this model"),
-                            FALSE,
-                            GTK_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY);
+  properties[PROP_SORTER] =
+      g_param_spec_object ("sorter",
+                            P_("Sorter"),
+                            P_("The sorter for this model"),
+                            GTK_TYPE_SORTER,
+                            GTK_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * GtkSortListModel:item-type:
@@ -348,36 +404,30 @@ gtk_sort_list_model_init (GtkSortListModel *self)
 {
 }
 
-
 /**
  * gtk_sort_list_model_new:
  * @model: the model to sort
- * @sort_func: (allow-none): sort function or %NULL to not sort items
- * @user_data: (closure): user data passed to @sort_func
- * @user_destroy: destroy notifier for @user_data
+ * @sorter: (allow-none): the #GtkSorter to sort @model with
  *
- * Creates a new sort list model that uses the @sort_func to sort @model.
+ * Creates a new sort list model that uses the @sorter to sort @model.
  *
  * Returns: a new #GtkSortListModel
  **/
 GtkSortListModel *
-gtk_sort_list_model_new (GListModel       *model,
-                         GCompareDataFunc  sort_func,
-                         gpointer          user_data,
-                         GDestroyNotify    user_destroy)
+gtk_sort_list_model_new (GListModel *model,
+                         GtkSorter  *sorter)
 {
   GtkSortListModel *result;
 
   g_return_val_if_fail (G_IS_LIST_MODEL (model), NULL);
+  g_return_val_if_fail (sorter == NULL || GTK_IS_SORTER (sorter), NULL);
 
   result = g_object_new (GTK_TYPE_SORT_LIST_MODEL,
                          "item-type", g_list_model_get_item_type (model),
                          "model", model,
+                         "sorter", sorter,
                          NULL);
 
-  if (sort_func)
-    gtk_sort_list_model_set_sort_func (result, sort_func, user_data, user_destroy);
-
   return result;
 }
 
@@ -404,59 +454,15 @@ gtk_sort_list_model_new_for_type (GType item_type)
 static void
 gtk_sort_list_model_create_sequences (GtkSortListModel *self)
 {
-  if (!self->sort_func || self->model == NULL)
+  if (self->sorter == NULL || self->model == NULL)
     return;
 
-  self->sorted = g_sequence_new (g_object_unref);
+  self->sorted = g_sequence_new (gtk_sort_list_entry_free);
   self->unsorted = g_sequence_new (NULL);
 
   gtk_sort_list_model_add_items (self, 0, g_list_model_get_n_items (self->model), NULL, NULL);
 }
 
-/**
- * gtk_sort_list_model_set_sort_func:
- * @self: a #GtkSortListModel
- * @sort_func: (allow-none): sort function or %NULL to not sort items
- * @user_data: (closure): user data passed to @sort_func
- * @user_destroy: destroy notifier for @user_data
- *
- * Sets the function used to sort items. The function will be called for every
- * item and must return an integer less than, equal to, or greater than zero if
- * for two items from the model if the first item is considered to be respectively
- * less than, equal to, or greater than the second.
- **/
-void
-gtk_sort_list_model_set_sort_func (GtkSortListModel *self,
-                                   GCompareDataFunc  sort_func,
-                                   gpointer          user_data,
-                                   GDestroyNotify    user_destroy)
-{
-  guint n_items;
-
-  g_return_if_fail (GTK_IS_SORT_LIST_MODEL (self));
-  g_return_if_fail (sort_func != NULL || (user_data == NULL && !user_destroy));
-
-  if (!sort_func && !self->sort_func)
-    return;
-
-  if (self->user_destroy)
-    self->user_destroy (self->user_data);
-
-  g_clear_pointer (&self->unsorted, g_sequence_free);
-  g_clear_pointer (&self->sorted, g_sequence_free);
-  self->sort_func = sort_func;
-  self->user_data = user_data;
-  self->user_destroy = user_destroy;
-  
-    gtk_sort_list_model_create_sequences (self);
-    
-  n_items = g_list_model_get_n_items (G_LIST_MODEL (self));
-  if (n_items > 1)
-    g_list_model_items_changed (G_LIST_MODEL (self), 0, n_items, n_items);
-
-  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_HAS_SORT]);
-}
-
 /**
  * gtk_sort_list_model_set_model:
  * @self: a #GtkSortListModel
@@ -517,32 +523,7 @@ gtk_sort_list_model_get_model (GtkSortListModel *self)
   return self->model;
 }
 
-/**
- * gtk_sort_list_model_has_sort:
- * @self: a #GtkSortListModel
- *
- * Checks if a sort function is currently set on @self
- *
- * Returns: %TRUE if a sort function is set
- **/
-gboolean
-gtk_sort_list_model_has_sort (GtkSortListModel *self)
-{
-  g_return_val_if_fail (GTK_IS_SORT_LIST_MODEL (self), FALSE);
-
-  return self->sort_func != NULL;
-}
-
-/**
- * gtk_sort_list_model_resort:
- * @self: a #GtkSortListModel
- *
- * Causes @self to resort all items in the model.
- *
- * Calling this function is necessary when data used by the sort
- * function has changed.
- **/
-void
+static void
 gtk_sort_list_model_resort (GtkSortListModel *self)
 {
   guint n_items;
@@ -556,8 +537,59 @@ gtk_sort_list_model_resort (GtkSortListModel *self)
   if (n_items <= 1)
     return;
 
-  g_sequence_sort (self->sorted, self->sort_func, self->user_data);
+  g_sequence_sort (self->sorted, _sort_func, self->sorter);
 
   g_list_model_items_changed (G_LIST_MODEL (self), 0, n_items, n_items);
 }
 
+/**
+ * gtk_sort_list_model_set_sorter:
+ * @self: a #GtkSortListModel
+ * @sorter: (allow-none): the #GtkSorter to sort @model with
+ *
+ * Sets a new sorter on @self.
+ */
+void
+gtk_sort_list_model_set_sorter (GtkSortListModel *self,
+                                GtkSorter        *sorter)
+{
+  guint n_items;
+
+  g_return_if_fail (GTK_IS_SORT_LIST_MODEL (self));
+  g_return_if_fail (sorter == NULL || GTK_IS_SORTER (sorter));
+
+  gtk_sort_list_model_clear_sorter (self);
+
+  if (sorter)
+    {
+      self->sorter = g_object_ref (sorter);
+      g_signal_connect (sorter, "changed", G_CALLBACK (gtk_sort_list_model_sorter_changed_cb), self);
+    }
+
+  g_clear_pointer (&self->unsorted, g_sequence_free);
+  g_clear_pointer (&self->sorted, g_sequence_free);
+  
+  gtk_sort_list_model_create_sequences (self);
+    
+  n_items = g_list_model_get_n_items (G_LIST_MODEL (self));
+  if (n_items > 1)
+    g_list_model_items_changed (G_LIST_MODEL (self), 0, n_items, n_items);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SORTER]);
+}
+
+/**
+ * gtk_sort_list_model_get_sorter:
+ * @self: a #GtkSortLisTModel
+ *
+ * Gets the sorter that is used to sort @self.
+ *
+ * Returns: (nullable) (transfer none): the sorter of #self
+ */
+GtkSorter *
+gtk_sort_list_model_get_sorter (GtkSortListModel *self)
+{
+  g_return_val_if_fail (GTK_IS_SORT_LIST_MODEL (self), NULL);
+
+  return self->sorter;
+}
