@@ -29,7 +29,7 @@
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
-#include <gtk/gtkcontainer.h>
+#include <gtk/gtkwidget.h>
 #include <gtk/gtkimcontext.h>
 #include <gtk/gtktextbuffer.h>
 
@@ -107,7 +107,7 @@ typedef struct _GtkTextViewClass   GtkTextViewClass;
 
 struct _GtkTextView
 {
-  GtkContainer parent_instance;
+  GtkWidget parent_instance;
 
   /*< private >*/
 
@@ -129,7 +129,7 @@ struct _GtkTextView
  *   keybinding signal.
  * @cut_clipboard: The class handler for the #GtkTextView::cut-clipboard
  *   keybinding signal
- * @copy_clipboard: The class handler for the #GtkTextview::copy-clipboard
+ * @copy_clipboard: The class handler for the #GtkTextView::copy-clipboard
  *   keybinding signal.
  * @paste_clipboard: The class handler for the #GtkTextView::paste-clipboard
  *   keybinding signal.
@@ -150,7 +150,7 @@ struct _GtkTextView
  */
 struct _GtkTextViewClass
 {
-  GtkContainerClass parent_class;
+  GtkWidgetClass parent_class;
 
   /*< public >*/
 
@@ -329,6 +329,10 @@ void       gtk_text_view_move_overlay        (GtkTextView          *text_view,
                                               gint                  xpos,
                                               gint                  ypos);
 
+GDK_AVAILABLE_IN_ALL
+void       gtk_text_view_remove              (GtkTextView          *text_view,
+                                              GtkWidget            *child);
+
 /* Default style settings (fallbacks if no tag affects the property) */
 
 GDK_AVAILABLE_IN_ALL
