@@ -269,6 +269,7 @@ gtk_tree_rbtree_remove_node_fixup (GtkTreeRBTree *tree,
               gtk_tree_rbnode_rotate_left (tree, parent);
               w = parent->right;
             }
+          g_assert (w);
           if (GTK_TREE_RBNODE_GET_COLOR (w->left) == GTK_TREE_RBNODE_BLACK && GTK_TREE_RBNODE_GET_COLOR (w->right) == GTK_TREE_RBNODE_BLACK)
             {
               GTK_TREE_RBNODE_SET_COLOR (w, GTK_TREE_RBNODE_RED);
@@ -300,6 +301,7 @@ gtk_tree_rbtree_remove_node_fixup (GtkTreeRBTree *tree,
               gtk_tree_rbnode_rotate_right (tree, parent);
               w = parent->left;
             }
+          g_assert (w);
           if (GTK_TREE_RBNODE_GET_COLOR (w->right) == GTK_TREE_RBNODE_BLACK && GTK_TREE_RBNODE_GET_COLOR (w->left) == GTK_TREE_RBNODE_BLACK)
             {
               GTK_TREE_RBNODE_SET_COLOR (w, GTK_TREE_RBNODE_RED);
@@ -722,9 +724,9 @@ gtk_tree_rbtree_column_invalid (GtkTreeRBTree *tree)
   if (tree == NULL)
     return;
 
-  node = gtk_tree_rbtree_first (tree);
-
-  do
+  for (node = gtk_tree_rbtree_first (tree);
+       node != NULL;
+       node = gtk_tree_rbtree_next (tree, node))
     {
       if (!(GTK_TREE_RBNODE_FLAG_SET (node, GTK_TREE_RBNODE_INVALID)))
         GTK_TREE_RBNODE_SET_FLAG (node, GTK_TREE_RBNODE_COLUMN_INVALID);
@@ -733,7 +735,6 @@ gtk_tree_rbtree_column_invalid (GtkTreeRBTree *tree)
       if (node->children)
         gtk_tree_rbtree_column_invalid (node->children);
     }
-  while ((node = gtk_tree_rbtree_next (tree, node)) != NULL);
 }
 
 void
@@ -744,9 +745,9 @@ gtk_tree_rbtree_mark_invalid (GtkTreeRBTree *tree)
   if (tree == NULL)
     return;
 
-  node = gtk_tree_rbtree_first (tree);
-
-  do
+  for (node = gtk_tree_rbtree_first (tree);
+       node != NULL;
+       node = gtk_tree_rbtree_next (tree, node))
     {
       GTK_TREE_RBNODE_SET_FLAG (node, GTK_TREE_RBNODE_INVALID);
       GTK_TREE_RBNODE_SET_FLAG (node, GTK_TREE_RBNODE_DESCENDANTS_INVALID);
@@ -754,7 +755,6 @@ gtk_tree_rbtree_mark_invalid (GtkTreeRBTree *tree)
       if (node->children)
         gtk_tree_rbtree_mark_invalid (node->children);
     }
-  while ((node = gtk_tree_rbtree_next (tree, node)) != NULL);
 }
 
 void
@@ -767,9 +767,9 @@ gtk_tree_rbtree_set_fixed_height (GtkTreeRBTree *tree,
   if (tree == NULL)
     return;
 
-  node = gtk_tree_rbtree_first (tree);
-
-  do
+  for (node = gtk_tree_rbtree_first (tree);
+       node != NULL;
+       node = gtk_tree_rbtree_next (tree, node))
     {
       if (GTK_TREE_RBNODE_FLAG_SET (node, GTK_TREE_RBNODE_INVALID))
         {
@@ -781,7 +781,6 @@ gtk_tree_rbtree_set_fixed_height (GtkTreeRBTree *tree,
       if (node->children)
         gtk_tree_rbtree_set_fixed_height (node->children, height, mark_valid);
     }
-  while ((node = gtk_tree_rbtree_next (tree, node)) != NULL);
 }
 
 static void
@@ -1598,6 +1597,8 @@ gtk_tree_rbtree_test_dirty (GtkTreeRBTree *tree,
                             GtkTreeRBNode *node,
                             gint           expected_dirtyness)
 {
+  g_assert (node);
+
   if (expected_dirtyness)
     {
       g_assert (GTK_TREE_RBNODE_FLAG_SET (node, GTK_TREE_RBNODE_COLUMN_INVALID) ||
