@@ -34,6 +34,7 @@
 #include "gtkstylecontext.h"
 #include "gtktypebuiltins.h"
 #include "gtkwidgetprivate.h"
+#include "gtkbuildable.h"
 #include "gtktext.h"
 
 
@@ -69,11 +70,10 @@ typedef struct _GtkViewportClass         GtkViewportClass;
 
 struct _GtkViewport
 {
-  GtkBin parent_instance;
-};
+  GtkWidget parent_instance;
+
+  GtkWidget *child;
 
-struct _GtkViewportPrivate
-{
   GtkAdjustment  *hadjustment;
   GtkAdjustment  *vadjustment;
  
@@ -88,7 +88,7 @@ struct _GtkViewportPrivate
 
 struct _GtkViewportClass
 {
-  GtkBinClass parent_class;
+  GtkWidgetClass parent_class;
 };
 
 enum {
@@ -97,25 +97,27 @@ enum {
   PROP_VADJUSTMENT,
   PROP_HSCROLL_POLICY,
   PROP_VSCROLL_POLICY,
-  PROP_SCROLL_TO_FOCUS
+  PROP_SCROLL_TO_FOCUS,
+  PROP_CHILD
 };
 
 
 static void gtk_viewport_set_property             (GObject         *object,
-						   guint            prop_id,
-						   const GValue    *value,
-						   GParamSpec      *pspec);
+                                                   guint            prop_id,
+                                                   const GValue    *value,
+                                                   GParamSpec      *pspec);
 static void gtk_viewport_get_property             (GObject         *object,
-						   guint            prop_id,
-						   GValue          *value,
-						   GParamSpec      *pspec);
-static void gtk_viewport_destroy                  (GtkWidget        *widget);
-static void gtk_viewport_size_allocate            (GtkWidget        *widget,
-                                                   int               width,
-                                                   int               height,
-                                                   int               baseline);
+                                                   guint            prop_id,
+                                                   GValue          *value,
+                                                   GParamSpec      *pspec);
+static void gtk_viewport_dispose                  (GObject         *object);
+static void gtk_viewport_size_allocate            (GtkWidget       *widget,
+                                                   int              width,
+                                                   int              height,
+                                                   int              baseline);
+
 static void gtk_viewport_adjustment_value_changed (GtkAdjustment    *adjustment,
-						   gpointer          data);
+                                                   gpointer          data);
 static void viewport_set_adjustment               (GtkViewport      *viewport,
                                                    GtkOrientation    orientation,
                                                    GtkAdjustment    *adjustment);
@@ -123,21 +125,44 @@ static void viewport_set_adjustment               (GtkViewport      *viewport,
 static void setup_focus_change_handler (GtkViewport *viewport);
 static void clear_focus_change_handler (GtkViewport *viewport);
 
-G_DEFINE_TYPE_WITH_CODE (GtkViewport, gtk_viewport, GTK_TYPE_BIN,
-                         G_ADD_PRIVATE (GtkViewport)
+static void gtk_viewport_buildable_init (GtkBuildableIface *iface);
+
+
+G_DEFINE_TYPE_WITH_CODE (GtkViewport, gtk_viewport, GTK_TYPE_WIDGET,
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
+                                                gtk_viewport_buildable_init)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_SCROLLABLE, NULL))
 
+static GtkBuildableIface *parent_buildable_iface;
+
+static void
+gtk_viewport_buildable_add_child (GtkBuildable *buildable,
+                                  GtkBuilder   *builder,
+                                  GObject      *child,
+                                  const gchar  *type)
+{
+  if (GTK_IS_WIDGET (child))
+    gtk_viewport_set_child (GTK_VIEWPORT (buildable), GTK_WIDGET (child));
+  else
+    parent_buildable_iface->add_child (buildable, builder, child, type);
+}
+
+static void
+gtk_viewport_buildable_init (GtkBuildableIface *iface)
+{
+  parent_buildable_iface = g_type_interface_peek_parent (iface);
+
+  iface->add_child = gtk_viewport_buildable_add_child;
+}
+
 static void
 viewport_set_adjustment_values (GtkViewport    *viewport,
                                 GtkOrientation  orientation)
 {
-  GtkBin *bin = GTK_BIN (viewport);
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
   GtkAdjustment *adjustment;
   GtkScrollablePolicy scroll_policy;
   GtkScrollablePolicy other_scroll_policy;
   GtkOrientation other_orientation;
-  GtkWidget *child;
   gdouble upper, value;
   int viewport_size, other_viewport_size;
   int view_width, view_height;
@@ -147,38 +172,37 @@ viewport_set_adjustment_values (GtkViewport    *viewport,
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
     {
-      adjustment = priv->hadjustment;
+      adjustment = viewport->hadjustment;
       other_orientation = GTK_ORIENTATION_VERTICAL;
       viewport_size = view_width;
       other_viewport_size = view_height;
-      scroll_policy = priv->hscroll_policy;
-      other_scroll_policy = priv->vscroll_policy;
+      scroll_policy = viewport->hscroll_policy;
+      other_scroll_policy = viewport->vscroll_policy;
     }
   else /* VERTICAL */
     {
-      adjustment = priv->vadjustment;
+      adjustment = viewport->vadjustment;
       other_orientation = GTK_ORIENTATION_HORIZONTAL;
       viewport_size = view_height;
       other_viewport_size = view_width;
-      scroll_policy = priv->vscroll_policy;
-      other_scroll_policy = priv->hscroll_policy;
+      scroll_policy = viewport->vscroll_policy;
+      other_scroll_policy = viewport->hscroll_policy;
     }
 
 
-  child = gtk_bin_get_child (bin);
-  if (child && gtk_widget_get_visible (child))
+  if (viewport->child && gtk_widget_get_visible (viewport->child))
     {
       int min_size, nat_size;
       int scroll_size;
 
       if (other_scroll_policy == GTK_SCROLL_MINIMUM)
-        gtk_widget_measure (child, other_orientation, -1,
+        gtk_widget_measure (viewport->child, other_orientation, -1,
                             &scroll_size, NULL, NULL, NULL);
       else
-        gtk_widget_measure (child, other_orientation, -1,
+        gtk_widget_measure (viewport->child, other_orientation, -1,
                             NULL, &scroll_size, NULL, NULL);
 
-      gtk_widget_measure (child, orientation,
+      gtk_widget_measure (viewport->child, orientation,
                           MAX (other_viewport_size, scroll_size),
                           &min_size, &nat_size, NULL, NULL);
 
@@ -223,23 +247,79 @@ gtk_viewport_measure (GtkWidget      *widget,
                       int            *minimum_baseline,
                       int            *natural_baseline)
 {
-  GtkWidget *child;
+  GtkViewport *viewport = GTK_VIEWPORT (widget);
 
   *minimum = *natural = 0;
 
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child && gtk_widget_get_visible (child))
-    gtk_widget_measure (child,
+  if (viewport->child && gtk_widget_get_visible (viewport->child))
+    gtk_widget_measure (viewport->child,
                         orientation,
                         for_size,
                         minimum, natural,
                         NULL, NULL);
 }
 
+static void
+gtk_viewport_compute_expand (GtkWidget *widget,
+                             gboolean  *hexpand,
+                             gboolean  *vexpand)
+{
+  GtkViewport *viewport = GTK_VIEWPORT (widget);
+
+  if (viewport->child)
+    {
+      *hexpand = gtk_widget_compute_expand (viewport->child, GTK_ORIENTATION_HORIZONTAL);
+      *vexpand = gtk_widget_compute_expand (viewport->child, GTK_ORIENTATION_VERTICAL);
+    }
+  else
+    {
+      *hexpand = FALSE;
+      *vexpand = FALSE;
+    }
+}
+
+static GtkSizeRequestMode
+gtk_viewport_get_request_mode (GtkWidget *widget)
+{
+  GtkViewport *viewport = GTK_VIEWPORT (widget);
+
+  if (viewport->child)
+    return gtk_widget_get_request_mode (viewport->child);
+  else
+    return GTK_SIZE_REQUEST_CONSTANT_SIZE;
+}
+
+#define ADJUSTMENT_POINTER(orientation)            \
+  (((orientation) == GTK_ORIENTATION_HORIZONTAL) ? \
+     &viewport->hadjustment : &viewport->vadjustment)
+
+static void
+viewport_disconnect_adjustment (GtkViewport    *viewport,
+                                GtkOrientation  orientation)
+{
+  GtkAdjustment **adjustmentp = ADJUSTMENT_POINTER (orientation);
+
+  if (*adjustmentp)
+    {
+      g_signal_handlers_disconnect_by_func (*adjustmentp,
+                                            gtk_viewport_adjustment_value_changed,
+                                            viewport);
+      g_object_unref (*adjustmentp);
+      *adjustmentp = NULL;
+    }
+}
+
 static void
 gtk_viewport_dispose (GObject *object)
 {
-  clear_focus_change_handler (GTK_VIEWPORT (object));
+  GtkViewport *viewport = GTK_VIEWPORT (object);
+
+  viewport_disconnect_adjustment (viewport, GTK_ORIENTATION_HORIZONTAL);
+  viewport_disconnect_adjustment (viewport, GTK_ORIENTATION_VERTICAL);
+
+  clear_focus_change_handler (viewport);
+
+  g_clear_pointer (&viewport->child, gtk_widget_unparent);
 
   G_OBJECT_CLASS (gtk_viewport_parent_class)->dispose (object);
 
@@ -249,11 +329,10 @@ static void
 gtk_viewport_root (GtkWidget *widget)
 {
   GtkViewport *viewport = GTK_VIEWPORT (widget);
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
 
   GTK_WIDGET_CLASS (gtk_viewport_parent_class)->root (widget);
 
-  if (priv->scroll_to_focus)
+  if (viewport->scroll_to_focus)
     setup_focus_change_handler (viewport);
 }
 
@@ -261,9 +340,8 @@ static void
 gtk_viewport_unroot (GtkWidget *widget)
 {
   GtkViewport *viewport = GTK_VIEWPORT (widget);
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
 
-  if (priv->scroll_to_focus)
+  if (viewport->scroll_to_focus)
     clear_focus_change_handler (viewport);
 
   GTK_WIDGET_CLASS (gtk_viewport_parent_class)->unroot (widget);
@@ -282,12 +360,13 @@ gtk_viewport_class_init (GtkViewportClass *class)
   gobject_class->set_property = gtk_viewport_set_property;
   gobject_class->get_property = gtk_viewport_get_property;
 
-  widget_class->destroy = gtk_viewport_destroy;
   widget_class->size_allocate = gtk_viewport_size_allocate;
   widget_class->measure = gtk_viewport_measure;
   widget_class->root = gtk_viewport_root;
   widget_class->unroot = gtk_viewport_unroot;
-  
+  widget_class->compute_expand = gtk_viewport_compute_expand;
+  widget_class->get_request_mode = gtk_viewport_get_request_mode;
+
   gtk_widget_class_set_accessible_role (widget_class, ATK_ROLE_VIEWPORT);
 
   /* GtkScrollable implementation */
@@ -304,17 +383,25 @@ gtk_viewport_class_init (GtkViewportClass *class)
                                                          FALSE,
                                                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_CHILD,
+                                   g_param_spec_object ("child",
+                                                        P_("Child"),
+                                                        P_("The child widget"),
+                                                        GTK_TYPE_WIDGET,
+                                                        GTK_PARAM_READWRITE));
+
   gtk_widget_class_set_css_name (widget_class, I_("viewport"));
 }
 
 static void
 gtk_viewport_set_property (GObject         *object,
-			   guint            prop_id,
-			   const GValue    *value,
-			   GParamSpec      *pspec)
+                           guint            prop_id,
+                           const GValue    *value,
+                           GParamSpec      *pspec)
 {
   GtkViewport *viewport = GTK_VIEWPORT (object);
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
 
   switch (prop_id)
     {
@@ -325,17 +412,17 @@ gtk_viewport_set_property (GObject         *object,
       viewport_set_adjustment (viewport, GTK_ORIENTATION_VERTICAL, g_value_get_object (value));
       break;
     case PROP_HSCROLL_POLICY:
-      if (priv->hscroll_policy != g_value_get_enum (value))
+      if (viewport->hscroll_policy != g_value_get_enum (value))
         {
-          priv->hscroll_policy = g_value_get_enum (value);
+          viewport->hscroll_policy = g_value_get_enum (value);
           gtk_widget_queue_resize (GTK_WIDGET (viewport));
           g_object_notify_by_pspec (object, pspec);
         }
       break;
     case PROP_VSCROLL_POLICY:
-      if (priv->vscroll_policy != g_value_get_enum (value))
+      if (viewport->vscroll_policy != g_value_get_enum (value))
         {
-          priv->vscroll_policy = g_value_get_enum (value);
+          viewport->vscroll_policy = g_value_get_enum (value);
           gtk_widget_queue_resize (GTK_WIDGET (viewport));
           g_object_notify_by_pspec (object, pspec);
         }
@@ -343,6 +430,9 @@ gtk_viewport_set_property (GObject         *object,
     case PROP_SCROLL_TO_FOCUS:
       gtk_viewport_set_scroll_to_focus (viewport, g_value_get_boolean (value));
       break;
+    case PROP_CHILD:
+      gtk_viewport_set_child (viewport, g_value_get_object (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -351,29 +441,31 @@ gtk_viewport_set_property (GObject         *object,
 
 static void
 gtk_viewport_get_property (GObject         *object,
-			   guint            prop_id,
-			   GValue          *value,
-			   GParamSpec      *pspec)
+                           guint            prop_id,
+                           GValue          *value,
+                           GParamSpec      *pspec)
 {
   GtkViewport *viewport = GTK_VIEWPORT (object);
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
 
   switch (prop_id)
     {
     case PROP_HADJUSTMENT:
-      g_value_set_object (value, priv->hadjustment);
+      g_value_set_object (value, viewport->hadjustment);
       break;
     case PROP_VADJUSTMENT:
-      g_value_set_object (value, priv->vadjustment);
+      g_value_set_object (value, viewport->vadjustment);
       break;
     case PROP_HSCROLL_POLICY:
-      g_value_set_enum (value, priv->hscroll_policy);
+      g_value_set_enum (value, viewport->hscroll_policy);
       break;
     case PROP_VSCROLL_POLICY:
-      g_value_set_enum (value, priv->vscroll_policy);
+      g_value_set_enum (value, viewport->vscroll_policy);
       break;
     case PROP_SCROLL_TO_FOCUS:
-      g_value_set_boolean (value, priv->scroll_to_focus);
+      g_value_set_boolean (value, viewport->scroll_to_focus);
+      break;
+    case PROP_CHILD:
+      g_value_set_object (value, gtk_viewport_get_child (viewport));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -385,14 +477,13 @@ static void
 gtk_viewport_init (GtkViewport *viewport)
 {
   GtkWidget *widget;
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
 
   widget = GTK_WIDGET (viewport);
 
   gtk_widget_set_overflow (widget, GTK_OVERFLOW_HIDDEN);
 
-  priv->hadjustment = NULL;
-  priv->vadjustment = NULL;
+  viewport->hadjustment = NULL;
+  viewport->vadjustment = NULL;
 
   gtk_widget_add_css_class (widget, GTK_STYLE_CLASS_FRAME);
   viewport_set_adjustment (viewport, GTK_ORIENTATION_HORIZONTAL, NULL);
@@ -411,7 +502,7 @@ gtk_viewport_init (GtkViewport *viewport)
  */
 GtkWidget*
 gtk_viewport_new (GtkAdjustment *hadjustment,
-		  GtkAdjustment *vadjustment)
+                  GtkAdjustment *vadjustment)
 {
   GtkWidget *viewport;
 
@@ -423,44 +514,11 @@ gtk_viewport_new (GtkAdjustment *hadjustment,
   return viewport;
 }
 
-#define ADJUSTMENT_POINTER(orientation)            \
-  (((orientation) == GTK_ORIENTATION_HORIZONTAL) ? \
-     &priv->hadjustment : &priv->vadjustment)
-
-static void
-viewport_disconnect_adjustment (GtkViewport    *viewport,
-				GtkOrientation  orientation)
-{
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
-  GtkAdjustment **adjustmentp = ADJUSTMENT_POINTER (orientation);
-
-  if (*adjustmentp)
-    {
-      g_signal_handlers_disconnect_by_func (*adjustmentp,
-					    gtk_viewport_adjustment_value_changed,
-					    viewport);
-      g_object_unref (*adjustmentp);
-      *adjustmentp = NULL;
-    }
-}
-
-static void
-gtk_viewport_destroy (GtkWidget *widget)
-{
-  GtkViewport *viewport = GTK_VIEWPORT (widget);
-
-  viewport_disconnect_adjustment (viewport, GTK_ORIENTATION_HORIZONTAL);
-  viewport_disconnect_adjustment (viewport, GTK_ORIENTATION_VERTICAL);
-
-  GTK_WIDGET_CLASS (gtk_viewport_parent_class)->destroy (widget);
-}
-
 static void
 viewport_set_adjustment (GtkViewport    *viewport,
-			 GtkOrientation  orientation,
-			 GtkAdjustment  *adjustment)
+                         GtkOrientation  orientation,
+                         GtkAdjustment  *adjustment)
 {
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
   GtkAdjustment **adjustmentp = ADJUSTMENT_POINTER (orientation);
 
   if (adjustment && adjustment == *adjustmentp)
@@ -475,8 +533,8 @@ viewport_set_adjustment (GtkViewport    *viewport,
   viewport_set_adjustment_values (viewport, orientation);
 
   g_signal_connect (adjustment, "value-changed",
-		    G_CALLBACK (gtk_viewport_adjustment_value_changed),
-		    viewport);
+                    G_CALLBACK (gtk_viewport_adjustment_value_changed),
+                    viewport);
 
   gtk_viewport_adjustment_value_changed (adjustment, viewport);
 }
@@ -488,10 +546,8 @@ gtk_viewport_size_allocate (GtkWidget *widget,
                             int        baseline)
 {
   GtkViewport *viewport = GTK_VIEWPORT (widget);
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
-  GtkAdjustment *hadjustment = priv->hadjustment;
-  GtkAdjustment *vadjustment = priv->vadjustment;
-  GtkWidget *child;
+  GtkAdjustment *hadjustment = viewport->hadjustment;
+  GtkAdjustment *vadjustment = viewport->vadjustment;
 
   g_object_freeze_notify (G_OBJECT (hadjustment));
   g_object_freeze_notify (G_OBJECT (vadjustment));
@@ -499,8 +555,7 @@ gtk_viewport_size_allocate (GtkWidget *widget,
   viewport_set_adjustment_values (viewport, GTK_ORIENTATION_HORIZONTAL);
   viewport_set_adjustment_values (viewport, GTK_ORIENTATION_VERTICAL);
 
-  child = gtk_bin_get_child (GTK_BIN (widget));
-  if (child && gtk_widget_get_visible (child))
+  if (viewport->child && gtk_widget_get_visible (viewport->child))
     {
       GtkAllocation child_allocation;
 
@@ -509,7 +564,7 @@ gtk_viewport_size_allocate (GtkWidget *widget,
       child_allocation.width = gtk_adjustment_get_upper (hadjustment);
       child_allocation.height = gtk_adjustment_get_upper (vadjustment);
 
-      gtk_widget_size_allocate (child, &child_allocation, -1);
+      gtk_widget_size_allocate (viewport->child, &child_allocation, -1);
     }
 
   g_object_thaw_notify (G_OBJECT (hadjustment));
@@ -535,11 +590,9 @@ gtk_viewport_adjustment_value_changed (GtkAdjustment *adjustment,
 gboolean
 gtk_viewport_get_scroll_to_focus (GtkViewport *viewport)
 {
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
-
   g_return_val_if_fail (GTK_IS_VIEWPORT (viewport), FALSE);
 
-  return priv->scroll_to_focus;
+  return viewport->scroll_to_focus;
 }
 
 /**
@@ -554,14 +607,12 @@ void
 gtk_viewport_set_scroll_to_focus (GtkViewport *viewport,
                                   gboolean     scroll_to_focus)
 {
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
-
   g_return_if_fail (GTK_IS_VIEWPORT (viewport));
 
-  if (priv->scroll_to_focus == scroll_to_focus)
+  if (viewport->scroll_to_focus == scroll_to_focus)
     return;
 
-  priv->scroll_to_focus = scroll_to_focus;
+  viewport->scroll_to_focus = scroll_to_focus;
 
   if (gtk_widget_get_root (GTK_WIDGET (viewport)))
     {
@@ -594,12 +645,10 @@ static void
 focus_change_handler (GtkWidget *widget)
 {
   GtkViewport *viewport = GTK_VIEWPORT (widget);
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
   GtkRoot *root;
   GtkWidget *focus_widget;
-  GtkWidget *child;
   graphene_rect_t rect;
-  int x, y;
+  double x, y;
 
   if ((gtk_widget_get_state_flags (widget) & GTK_STATE_FLAG_FOCUS_WITHIN) == 0)
     return;
@@ -613,43 +662,84 @@ focus_change_handler (GtkWidget *widget)
   if (GTK_IS_TEXT (focus_widget))
     focus_widget = gtk_widget_get_parent (focus_widget);
 
-  child = gtk_bin_get_child (GTK_BIN (viewport));
-
-  if (!gtk_widget_compute_bounds (focus_widget, child, &rect))
+  if (!gtk_widget_compute_bounds (focus_widget, viewport->child, &rect))
     return;
 
-  gtk_widget_translate_coordinates (child, widget,
-                                    (int)rect.origin.x,
-                                    (int)rect.origin.y,
+  gtk_widget_translate_coordinates (viewport->child, widget,
+                                    rect.origin.x,
+                                    rect.origin.y,
                                      &x, &y);
 
-  scroll_to_view (priv->hadjustment, x, rect.size.width);
-  scroll_to_view (priv->vadjustment, y, rect.size.height);
+  scroll_to_view (viewport->hadjustment, x, rect.size.width);
+  scroll_to_view (viewport->vadjustment, y, rect.size.height);
 }
 
 static void
 setup_focus_change_handler (GtkViewport *viewport)
 {
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
   GtkRoot *root;
 
   root = gtk_widget_get_root (GTK_WIDGET (viewport));
 
-  priv->focus_handler = g_signal_connect_swapped (root, "notify::focus-widget",
+  viewport->focus_handler = g_signal_connect_swapped (root, "notify::focus-widget",
                                                   G_CALLBACK (focus_change_handler), viewport);
 }
 
 static void
 clear_focus_change_handler (GtkViewport *viewport)
 {
-  GtkViewportPrivate *priv = gtk_viewport_get_instance_private (viewport);
   GtkRoot *root;
 
   root = gtk_widget_get_root (GTK_WIDGET (viewport));
 
-  if (priv->focus_handler)
+  if (viewport->focus_handler)
     {
-      g_signal_handler_disconnect (root, priv->focus_handler);
-      priv->focus_handler = 0;
+      g_signal_handler_disconnect (root, viewport->focus_handler);
+      viewport->focus_handler = 0;
     }
 }
+
+/**
+ * gtk_viewport_set_child:
+ * @viewport: a #GtkViewport
+ * @child: (allow-none): the child widget
+ *
+ * Sets the child widget of @viewport.
+ */
+void
+gtk_viewport_set_child (GtkViewport *viewport,
+                        GtkWidget   *child)
+{
+  g_return_if_fail (GTK_IS_VIEWPORT (viewport));
+  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+
+  if (viewport->child == child)
+    return;
+
+  g_clear_pointer (&viewport->child, gtk_widget_unparent);
+
+  if (child)
+    {
+      viewport->child = child;
+      gtk_widget_set_parent (child, GTK_WIDGET (viewport));
+    }
+
+  g_object_notify (G_OBJECT (viewport), "child");
+}
+
+/**
+ * gtk_viewport_get_child:
+ * @viewport: a #GtkViewport
+ *
+ * Gets the child widget of @viewport.
+ *
+ * Returns: (nullable) (transfer none): the child widget of @viewport
+ */
+GtkWidget *
+gtk_viewport_get_child (GtkViewport *viewport)
+{
+  g_return_val_if_fail (GTK_IS_VIEWPORT (viewport), NULL);
+
+  return viewport->child;
+}
+
