@@ -51,7 +51,6 @@ void            _gtk_window_set_allocation         (GtkWindow     *window,
                                                     int            width,
                                                     int            height,
                                                     GtkAllocation *allocation_out);
-void            gtk_window_check_resize            (GtkWindow     *self);
 
 typedef void (*GtkWindowKeysForeachFunc) (GtkWindow      *window,
                                           guint           keyval,
@@ -59,18 +58,12 @@ typedef void (*GtkWindowKeysForeachFunc) (GtkWindow      *window,
                                           gpointer        data);
 
 gboolean gtk_window_emit_close_request (GtkWindow *window);
-gboolean gtk_window_configure    (GtkWindow         *window,
-                                  guint              width,
-                                  guint              height);
 
 /* --- internal (GtkAcceleratable) --- */
 void            _gtk_window_schedule_mnemonics_visible (GtkWindow *window);
 
 void            _gtk_window_notify_keys_changed (GtkWindow *window);
 
-void            _gtk_window_get_shadow_width (GtkWindow *window,
-                                              GtkBorder *border);
-
 void            _gtk_window_toggle_maximized (GtkWindow *window);
 
 void            _gtk_window_request_csd (GtkWindow *window);
@@ -132,10 +125,6 @@ GtkWidget *      gtk_window_pick_popover (GtkWindow   *window,
                                           double       y,
                                           GtkPickFlags flags);
 
-void             gtk_window_set_extra_input_region (GtkWindow      *window,
-                                                    cairo_region_t *region);
-
-
 G_END_DECLS
 
 #endif /* __GTK_WINDOW_PRIVATE_H__ */
