@@ -21,8 +21,9 @@
 #include "controllers.h"
 #include "object-tree.h"
 
-#include "gtksizegroup.h"
-#include "gtkcomboboxtext.h"
+#include "gtkbinlayout.h"
+#include "gtkdropdown.h"
+#include "gtkcustomsorter.h"
 #include "gtkflattenlistmodel.h"
 #include "gtkframe.h"
 #include "gtkgesture.h"
@@ -31,10 +32,26 @@
 #include "gtkmaplistmodel.h"
 #include "gtkpropertylookuplistmodelprivate.h"
 #include "gtkscrolledwindow.h"
+#include "gtksizegroup.h"
 #include "gtksortlistmodel.h"
-#include "gtkwidgetprivate.h"
 #include "gtkstack.h"
 #include "gtkstylecontext.h"
+#include "gtkwidgetprivate.h"
+
+struct _GtkInspectorControllers
+{
+  GtkWidget parent_instance;
+
+  GtkWidget *listbox;
+  GtkPropertyLookupListModel *model;
+  GtkSizeGroup *sizegroup;
+  GtkInspectorObjectTree *object_tree;
+};
+
+struct _GtkInspectorControllersClass
+{
+  GtkWidgetClass parent_class;
+};
 
 enum
 {
@@ -42,34 +59,25 @@ enum
   PROP_OBJECT_TREE
 };
 
-struct _GtkInspectorControllersPrivate
-{
-  GtkWidget *listbox;
-  GtkPropertyLookupListModel *model;
-  GtkSizeGroup *sizegroup;
-  GtkInspectorObjectTree *object_tree;
-};
-
-G_DEFINE_TYPE_WITH_PRIVATE (GtkInspectorControllers, gtk_inspector_controllers, GTK_TYPE_BOX)
+G_DEFINE_TYPE (GtkInspectorControllers, gtk_inspector_controllers, GTK_TYPE_WIDGET)
 
 static void
 row_activated (GtkListBox              *box,
                GtkListBoxRow           *row,
-               GtkInspectorControllers *sl)
+               GtkInspectorControllers *self)
 {
   GObject *controller;
   
   controller = G_OBJECT (g_object_get_data (G_OBJECT (row), "controller"));
-  gtk_inspector_object_tree_select_object (sl->priv->object_tree, controller);
+  gtk_inspector_object_tree_select_object (self->object_tree, controller);
 }
 
 static void
-gtk_inspector_controllers_init (GtkInspectorControllers *sl)
+gtk_inspector_controllers_init (GtkInspectorControllers *self)
 {
   GtkWidget *sw, *box;
 
-  sl->priv = gtk_inspector_controllers_get_instance_private (sl);
-  sl->priv->sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+  self->sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
 
@@ -84,26 +92,27 @@ gtk_inspector_controllers_init (GtkInspectorControllers *sl)
   gtk_widget_set_hexpand (box, TRUE);
   gtk_widget_set_vexpand (box, TRUE);
 
-  sl->priv->listbox = gtk_list_box_new ();
-  gtk_widget_add_css_class (sl->priv->listbox, "frame");
-  gtk_widget_set_halign (sl->priv->listbox, GTK_ALIGN_CENTER);
-  g_signal_connect (sl->priv->listbox, "row-activated", G_CALLBACK (row_activated), sl);
-  gtk_list_box_set_selection_mode (GTK_LIST_BOX (sl->priv->listbox), GTK_SELECTION_NONE);
-  gtk_box_append (GTK_BOX (box), sl->priv->listbox);
+  self->listbox = gtk_list_box_new ();
+  gtk_widget_add_css_class (self->listbox, "frame");
+  gtk_widget_set_halign (self->listbox, GTK_ALIGN_CENTER);
+  g_signal_connect (self->listbox, "row-activated", G_CALLBACK (row_activated), self);
+  gtk_list_box_set_selection_mode (GTK_LIST_BOX (self->listbox), GTK_SELECTION_NONE);
+  gtk_box_append (GTK_BOX (box), self->listbox);
 
-  gtk_box_append (GTK_BOX (sl), sw);
+  gtk_widget_set_parent (sw, GTK_WIDGET (self));
 }
 
 static void
-phase_changed_cb (GtkComboBox             *combo,
-                  GtkInspectorControllers *sl)
+phase_changed_cb (GtkDropDown             *dropdown,
+                  GParamSpec              *pspec,
+                  GtkInspectorControllers *self)
 {
   GtkWidget *row;
   GtkPropagationPhase phase;
   GtkEventController *controller;
 
-  phase = gtk_combo_box_get_active (combo);
-  row = gtk_widget_get_ancestor (GTK_WIDGET (combo), GTK_TYPE_LIST_BOX_ROW);
+  phase = gtk_drop_down_get_selected (dropdown);
+  row = gtk_widget_get_ancestor (GTK_WIDGET (dropdown), GTK_TYPE_LIST_BOX_ROW);
   controller = GTK_EVENT_CONTROLLER (g_object_get_data (G_OBJECT (row), "controller"));
   gtk_event_controller_set_propagation_phase (controller, phase);
 }
@@ -112,12 +121,13 @@ static GtkWidget *
 create_controller_widget (gpointer item,
                           gpointer user_data)
 {
+  GtkInspectorControllers *self = user_data;
   GtkEventController *controller = item;
-  GtkInspectorControllers *sl = user_data;
   GtkWidget *row;
   GtkWidget *box;
   GtkWidget *label;
-  GtkWidget *combo;
+  GtkWidget *dropdown;
+  const char *phases[5];
 
   row = gtk_list_box_row_new ();
   gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), FALSE);
@@ -130,22 +140,25 @@ create_controller_widget (gpointer item,
   label = gtk_label_new (G_OBJECT_TYPE_NAME (controller));
   g_object_set (label, "xalign", 0.0, NULL);
   gtk_box_append (GTK_BOX (box), label);
-  gtk_size_group_add_widget (sl->priv->sizegroup, label);
+  gtk_size_group_add_widget (self->sizegroup, label);
   gtk_widget_set_halign (label, GTK_ALIGN_START);
   gtk_widget_set_valign (label, GTK_ALIGN_BASELINE);
 
-  combo = gtk_combo_box_text_new ();
-  gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (combo), GTK_PHASE_NONE, C_("event phase", "None"));
-  gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (combo), GTK_PHASE_CAPTURE, C_("event phase", "Capture"));
-  gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (combo), GTK_PHASE_BUBBLE, C_("event phase", "Bubble"));
-  gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (combo), GTK_PHASE_TARGET, C_("event phase", "Target"));
-  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), gtk_event_controller_get_propagation_phase (controller));
-  gtk_box_append (GTK_BOX (box), combo);
+  phases[0] = C_("event phase", "None");
+  phases[1] = C_("event phase", "Capture");
+  phases[2] = C_("event phase", "Bubble");
+  phases[3] = C_("event phase", "Target");
+  phases[4] = NULL;
+
+  dropdown = gtk_drop_down_new ();
+  gtk_drop_down_set_from_strings (GTK_DROP_DOWN (dropdown), phases);
+  gtk_drop_down_set_selected (GTK_DROP_DOWN (dropdown), gtk_event_controller_get_propagation_phase (controller));
+  gtk_box_append (GTK_BOX (box), dropdown);
   gtk_widget_set_halign (label, GTK_ALIGN_END);
   gtk_widget_set_valign (label, GTK_ALIGN_BASELINE);
 
   g_object_set_data (G_OBJECT (row), "controller", controller);
-  g_signal_connect (combo, "changed", G_CALLBACK (phase_changed_cb), sl);
+  g_signal_connect (dropdown, "notify::selected", G_CALLBACK (phase_changed_cb), self);
 
   return row;
 }
@@ -208,18 +221,18 @@ compare_controllers (gconstpointer _first,
 }
 
 void
-gtk_inspector_controllers_set_object (GtkInspectorControllers *sl,
+gtk_inspector_controllers_set_object (GtkInspectorControllers *self,
                                       GObject                 *object)
 {
   GtkWidget *stack;
   GtkStackPage *page;
-  GtkInspectorControllersPrivate *priv = sl->priv;
   GtkMapListModel *map_model;
   GtkFlattenListModel *flatten_model;
   GtkSortListModel *sort_model;
+  GtkSorter *sorter;
 
-  stack = gtk_widget_get_parent (GTK_WIDGET (sl));
-  page = gtk_stack_get_page (GTK_STACK (stack), GTK_WIDGET (sl));
+  stack = gtk_widget_get_parent (GTK_WIDGET (self));
+  page = gtk_stack_get_page (GTK_STACK (stack), GTK_WIDGET (self));
 
   if (!GTK_IS_WIDGET (object))
     {
@@ -229,22 +242,22 @@ gtk_inspector_controllers_set_object (GtkInspectorControllers *sl,
 
   g_object_set (page, "visible", TRUE, NULL);
 
-  priv->model = gtk_property_lookup_list_model_new (GTK_TYPE_WIDGET, "parent");
-  gtk_property_lookup_list_model_set_object (priv->model, object);
+  self->model = gtk_property_lookup_list_model_new (GTK_TYPE_WIDGET, "parent");
+  gtk_property_lookup_list_model_set_object (self->model, object);
 
-  map_model = gtk_map_list_model_new (G_TYPE_LIST_MODEL, G_LIST_MODEL (priv->model), map_to_controllers, NULL, NULL);
-  g_object_unref (priv->model);
+  map_model = gtk_map_list_model_new (G_TYPE_LIST_MODEL, G_LIST_MODEL (self->model), map_to_controllers, NULL, NULL);
+  g_object_unref (self->model);
 
   flatten_model = gtk_flatten_list_model_new (GTK_TYPE_EVENT_CONTROLLER, G_LIST_MODEL (map_model));
 
-  sort_model = gtk_sort_list_model_new (G_LIST_MODEL (flatten_model),
-                                        compare_controllers,
-                                        NULL, NULL);
+  sorter = gtk_custom_sorter_new (compare_controllers, NULL, NULL);
+  sort_model = gtk_sort_list_model_new (G_LIST_MODEL (flatten_model), sorter);
+  g_object_unref (sorter);
 
-  gtk_list_box_bind_model (GTK_LIST_BOX (priv->listbox),
+  gtk_list_box_bind_model (GTK_LIST_BOX (self->listbox),
                            G_LIST_MODEL (sort_model),
                            create_controller_widget,
-                           sl,
+                           self,
                            NULL);
 
   g_object_unref (sort_model);
@@ -253,17 +266,17 @@ gtk_inspector_controllers_set_object (GtkInspectorControllers *sl,
 }
 
 static void
-get_property (GObject    *object,
-              guint       param_id,
-              GValue     *value,
-              GParamSpec *pspec)
+gtk_inspector_controllers_get_property (GObject    *object,
+                                        guint       param_id,
+                                        GValue     *value,
+                                        GParamSpec *pspec)
 {
-  GtkInspectorControllers *sl = GTK_INSPECTOR_CONTROLLERS (object);
+  GtkInspectorControllers *self = GTK_INSPECTOR_CONTROLLERS (object);
 
   switch (param_id)
     {
       case PROP_OBJECT_TREE:
-        g_value_take_object (value, sl->priv->object_tree);
+        g_value_take_object (value, self->object_tree);
         break;
 
       default:
@@ -273,17 +286,17 @@ get_property (GObject    *object,
 }
 
 static void
-set_property (GObject      *object,
-              guint         param_id,
-              const GValue *value,
-              GParamSpec   *pspec)
+gtk_inspector_controllers_set_property (GObject      *object,
+                                        guint         param_id,
+                                        const GValue *value,
+                                        GParamSpec   *pspec)
 {
-  GtkInspectorControllers *sl = GTK_INSPECTOR_CONTROLLERS (object);
+  GtkInspectorControllers *self = GTK_INSPECTOR_CONTROLLERS (object);
 
   switch (param_id)
     {
       case PROP_OBJECT_TREE:
-        sl->priv->object_tree = g_value_get_object (value);
+        self->object_tree = g_value_get_object (value);
         break;
 
       default:
@@ -292,17 +305,31 @@ set_property (GObject      *object,
     }
 }
 
+static void
+gtk_inspector_controllers_dispose (GObject *object)
+{
+  GtkInspectorControllers *self = GTK_INSPECTOR_CONTROLLERS (object);
+
+  gtk_widget_unparent (gtk_widget_get_first_child (GTK_WIDGET (self)));
+
+  G_OBJECT_CLASS (gtk_inspector_controllers_parent_class)->dispose (object);
+}
+
 static void
 gtk_inspector_controllers_class_init (GtkInspectorControllersClass *klass)
 {
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->get_property = get_property;
-  object_class->set_property = set_property;
+  object_class->get_property = gtk_inspector_controllers_get_property;
+  object_class->set_property = gtk_inspector_controllers_set_property;
+  object_class->dispose= gtk_inspector_controllers_dispose;
 
   g_object_class_install_property (object_class, PROP_OBJECT_TREE,
       g_param_spec_object ("object-tree", "Widget Tree", "Widget tree",
                            GTK_TYPE_WIDGET, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
 }
 
 // vim: set et sw=2 ts=2:
