@@ -103,16 +103,16 @@ show_node_prop_editor (NodePropEditor *npe)
   GtkWidget *editor;
 
   popover = gtk_popover_new ();
-  gtk_widget_set_parent (popover, GTK_WIDGET (npe->cnt->priv->node_tree));
+  gtk_widget_set_parent (popover, GTK_WIDGET (npe->cnt));
   gtk_popover_set_pointing_to (GTK_POPOVER (popover), &npe->rect);
 
   editor = gtk_inspector_prop_editor_new (G_OBJECT (npe->node), npe->prop_name, NULL);
 
-  gtk_container_add (GTK_CONTAINER (popover), editor);
+  gtk_popover_set_child (GTK_POPOVER (popover), editor);
 
   gtk_popover_popup (GTK_POPOVER (popover));
 
-  g_signal_connect (popover, "unmap", G_CALLBACK (gtk_widget_destroy), NULL);
+  g_signal_connect (popover, "unmap", G_CALLBACK (gtk_widget_unparent), NULL);
 }
 
 static void
