@@ -34,18 +34,22 @@
 #include "gtkbuildable.h"
 #include "gtkbutton.h"
 #include "gtkcelllayout.h"
+#include "gtkcolumnview.h"
 #include "gtkcomboboxprivate.h"
 #include "gtkfilterlistmodel.h"
+#include "gtkcustomfilter.h"
 #include "gtkflattenlistmodel.h"
 #include "gtkbuiltiniconprivate.h"
 #include "gtkiconview.h"
 #include "gtklabel.h"
-#include "gtklistbox.h"
+#include "gtklistitem.h"
 #include "gtkpopover.h"
 #include "gtksettings.h"
-#include "gtksizegroup.h"
+#include "gtksingleselection.h"
+#include "gtksignallistitemfactory.h"
 #include "gtktextview.h"
 #include "gtktogglebutton.h"
+#include "gtktreeexpander.h"
 #include "gtktreelistmodel.h"
 #include "gtktreeview.h"
 #include "gtktreeselection.h"
@@ -68,13 +72,11 @@ enum
 
 struct _GtkInspectorObjectTreePrivate
 {
-  GtkListBox *list;
+  GtkColumnView *list;
   GtkTreeListModel *tree_model;
+  GtkSingleSelection *selection;
   GtkWidget *search_bar;
   GtkWidget *search_entry;
-  GtkSizeGroup *type_size_group;
-  GtkSizeGroup *name_size_group;
-  GtkSizeGroup *label_size_group;
 };
 
 typedef struct _ObjectTreeClassFuncs ObjectTreeClassFuncs;
@@ -334,6 +336,29 @@ object_tree_tree_view_get_children (GObject *object)
   return G_LIST_MODEL (result);
 }
 
+static GListModel *
+object_tree_column_view_get_children (GObject *object)
+{
+  GtkColumnView *view = GTK_COLUMN_VIEW (object);
+  GListStore *result_list;
+  GtkFlattenListModel *result;
+  GListModel *columns, *sublist;
+
+  result_list = g_list_store_new (G_TYPE_LIST_MODEL);
+
+  columns = gtk_column_view_get_columns (view);
+  g_list_store_append (result_list, columns);
+
+  sublist = object_tree_widget_get_children (object);
+  g_list_store_append (result_list, sublist);
+  g_object_unref (sublist);
+
+  result = gtk_flatten_list_model_new (G_TYPE_OBJECT, G_LIST_MODEL (result_list));
+  g_object_unref (result_list);
+
+  return G_LIST_MODEL (result);
+}
+
 static GListModel *
 object_tree_icon_view_get_children (GObject *object)
 {
@@ -452,7 +477,7 @@ object_tree_text_tag_table_get_children (GObject *object)
 static GListModel *
 object_tree_application_get_children (GObject *object)
 {
-  return list_model_for_properties (object, (const char *[3]) { "app-menu", "menubar", NULL });
+  return list_model_for_properties (object, (const char *[2]) { "menubar", NULL });
 }
 
 static GObject *
@@ -497,6 +522,11 @@ static const ObjectTreeClassFuncs object_tree_class_funcs[] = {
     object_tree_widget_get_parent,
     object_tree_tree_view_get_children
   },
+  {
+    gtk_column_view_get_type,
+    object_tree_widget_get_parent,
+    object_tree_column_view_get_children
+  },
   {
     gtk_combo_box_get_type,
     object_tree_widget_get_parent,
@@ -666,15 +696,13 @@ gtk_inspector_object_tree_activate_object (GtkInspectorObjectTree *wt,
 }
 
 static void
-on_row_activated (GtkListBox             *box,
-                  GtkListBoxRow          *row,
+on_row_activated (GtkColumnView          *view,
+                  guint                   pos,
                   GtkInspectorObjectTree *wt)
 {
-  guint pos;
   GtkTreeListRow *item;
   GObject *object;
 
-  pos = gtk_list_box_row_get_index (row);
   item = g_list_model_get_item (G_LIST_MODEL (wt->priv->tree_model), pos);
   object = gtk_tree_list_row_get_item (item);
 
@@ -687,37 +715,31 @@ on_row_activated (GtkListBox             *box,
 GObject *
 gtk_inspector_object_tree_get_selected (GtkInspectorObjectTree *wt)
 {
-  GtkListBoxRow *selected_row;
-  guint selected_pos;
-  GtkTreeListRow *selected_row_item;
+  GtkTreeListRow *selected_item;
   GObject *object;
 
-  selected_row = gtk_list_box_get_selected_row (wt->priv->list);
-  if (selected_row == NULL)
+  selected_item = gtk_single_selection_get_selected_item (wt->priv->selection);
+  if (selected_item == NULL)
     return NULL;
 
-  selected_pos = gtk_list_box_row_get_index (selected_row);
-  selected_row_item = g_list_model_get_item (G_LIST_MODEL (wt->priv->tree_model), selected_pos);
-
-  object = gtk_tree_list_row_get_item (selected_row_item);
-  g_object_unref (selected_row_item);
+  object = gtk_tree_list_row_get_item (selected_item);
 
   g_object_unref (object); /* ahem */
   return object;
 }
 
 static void
-widget_mapped (GtkWidget     *widget,
-               GtkListBoxRow *row)
+widget_mapped (GtkWidget *widget,
+               GtkWidget *label)
 {
-  gtk_widget_remove_css_class (GTK_WIDGET (row), "dim-label");
+  gtk_widget_remove_css_class (widget, "dim-label");
 }
 
 static void
-widget_unmapped (GtkWidget     *widget,
-                 GtkListBoxRow *row)
+widget_unmapped (GtkWidget *widget,
+                 GtkWidget *label)
 {
-  gtk_widget_add_css_class (GTK_WIDGET (row), "dim-label");
+  gtk_widget_add_css_class (widget, "dim-label");
 }
 
 static gboolean
@@ -893,16 +915,10 @@ search (GtkInspectorObjectTree *wt,
   const char *text;
 
   text = gtk_editable_get_text (GTK_EDITABLE (priv->search_entry));
-  if (gtk_list_box_get_selected_row (priv->list))
-    {
-      selected = gtk_list_box_row_get_index (gtk_list_box_get_selected_row (priv->list));
-    }
-  else
-    {
-      selected = 0;
-      force_progress = FALSE;
-    }
+  selected = gtk_single_selection_get_selected (priv->selection);
   n = g_list_model_get_n_items (model);
+  if (selected >= n)
+    selected = 0;
 
   for (i = 0; i < n; i++)
     {
@@ -913,7 +929,7 @@ search (GtkInspectorObjectTree *wt,
         {
           if (match_object (child, text))
             {
-              gtk_list_box_select_row (priv->list, gtk_list_box_get_row_at_index (priv->list, row));
+              gtk_single_selection_set_selected (priv->selection, row);
               g_object_unref (child);
               g_object_unref (row_item);
               return TRUE;
@@ -977,93 +993,149 @@ stop_search (GtkWidget              *entry,
   gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (wt->priv->search_bar), FALSE);
 }
 
-static GtkWidget *
-gtk_inspector_object_tree_create_list_widget (gpointer row_item,
-                                              gpointer user_data)
+static void
+setup_type_cb (GtkSignalListItemFactory *factory,
+               GtkListItem              *list_item)
 {
-  GtkInspectorObjectTree *wt = user_data;
-  gpointer item;
-  GtkWidget *row, *box, *column, *child;
-  guint depth;
-
-  item = gtk_tree_list_row_get_item (row_item);
-
-  row = gtk_list_box_row_new ();
-  g_object_set_data_full (G_OBJECT (row), "make-sure-its-not-unreffed", g_object_ref (row_item), g_object_unref);
-  if (GTK_IS_WIDGET (item))
-    {
-      g_signal_connect_object (item, "map", G_CALLBACK (widget_mapped), row, 0);
-      g_signal_connect_object (item, "unmap", G_CALLBACK (widget_unmapped), row, 0);
-      if (!gtk_widget_get_mapped (item))
-        widget_unmapped (item, GTK_LIST_BOX_ROW (row));
-    }
-
-  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_list_box_row_set_child (GTK_LIST_BOX_ROW (row), box);
-
-  column = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_size_group_add_widget (wt->priv->type_size_group, column);
-  gtk_box_append (GTK_BOX (box), column);
+  GtkWidget *expander, *label;
 
   /* expander */
-  depth = gtk_tree_list_row_get_depth (row_item);
-  if (depth > 0)
+  expander = gtk_tree_expander_new ();
+  gtk_list_item_set_child (list_item, expander);
+
+  /* label */
+  label = gtk_label_new (NULL);
+  gtk_label_set_width_chars (GTK_LABEL (label), 30);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_tree_expander_set_child (GTK_TREE_EXPANDER (expander), label);
+}
+
+static void
+bind_type_cb (GtkSignalListItemFactory *factory,
+              GtkListItem              *list_item)
+{
+  GtkWidget *expander, *label;
+  GtkTreeListRow *list_row;
+  gpointer item;
+
+  list_row = gtk_list_item_get_item (list_item);
+  expander = gtk_list_item_get_child (list_item);
+  gtk_tree_expander_set_list_row (GTK_TREE_EXPANDER (expander), list_row);
+  item = gtk_tree_list_row_get_item (list_row);
+  expander = gtk_list_item_get_child (list_item);
+  label = gtk_tree_expander_get_child (GTK_TREE_EXPANDER (expander));
+
+  gtk_label_set_label (GTK_LABEL (label), G_OBJECT_TYPE_NAME (item));
+
+  if (GTK_IS_WIDGET (item))
     {
-      child = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-      gtk_widget_set_size_request (child, 16 * depth, 0);
-      gtk_box_append (GTK_BOX (column), child);
+      g_signal_connect (item, "map", G_CALLBACK (widget_mapped), label);
+      g_signal_connect (item, "unmap", G_CALLBACK (widget_unmapped), label);
+      if (!gtk_widget_get_mapped (item))
+        widget_unmapped (item, label);
+      g_object_set_data (G_OBJECT (label), "binding", g_object_ref (item));
     }
-  if (gtk_tree_list_row_is_expandable (row_item))
+
+  g_object_unref (item);
+}
+
+static void
+unbind_type_cb (GtkSignalListItemFactory *factory,
+                GtkListItem              *list_item)
+{
+  GtkWidget *expander, *label;
+  gpointer item;
+
+  expander = gtk_list_item_get_child (list_item);
+  label = gtk_tree_expander_get_child (GTK_TREE_EXPANDER (expander));
+  item = g_object_steal_data (G_OBJECT (label), "binding");
+  if (item)
     {
-      GtkWidget *title, *arrow;
+      g_signal_handlers_disconnect_by_func (item, widget_mapped, label);
+      g_signal_handlers_disconnect_by_func (item, widget_unmapped, label);
 
-      child = g_object_new (GTK_TYPE_BOX, "css-name", "expander", NULL);
-
-      title = g_object_new (GTK_TYPE_TOGGLE_BUTTON, "css-name", "title", NULL);
-      gtk_button_set_has_frame (GTK_BUTTON (title), FALSE);
-      g_object_bind_property (row_item, "expanded", title, "active", G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-      gtk_box_append (GTK_BOX (child), title);
-
-      arrow = gtk_builtin_icon_new ("expander");
-      gtk_button_set_child (GTK_BUTTON (title), arrow);
+      g_object_unref (item);
     }
-  else
-    {
-      child = gtk_image_new (); /* empty whatever */
-    }
-  gtk_box_append (GTK_BOX (column), child);
+}
 
-  /* 1st column: type name */
-  child = gtk_label_new (G_OBJECT_TYPE_NAME (item));
-  gtk_label_set_width_chars (GTK_LABEL (child), 30);
-  gtk_label_set_xalign (GTK_LABEL (child), 0.0);
-  gtk_box_append (GTK_BOX (column), child);
+static void
+setup_name_cb (GtkSignalListItemFactory *factory,
+               GtkListItem              *list_item)
+{
+  GtkWidget *label;
 
-  /* 2nd column: name */
-  child = gtk_label_new (gtk_inspector_get_object_name (item));
-  gtk_label_set_width_chars (GTK_LABEL (child), 15);
-  gtk_label_set_xalign (GTK_LABEL (child), 0.0);
-  gtk_size_group_add_widget (wt->priv->name_size_group, child);
-  gtk_box_append (GTK_BOX (box), child);
+  label = gtk_label_new (NULL);
+  gtk_label_set_width_chars (GTK_LABEL (label), 15);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_list_item_set_child (list_item, label);
+}
+
+static void
+bind_name_cb (GtkSignalListItemFactory *factory,
+              GtkListItem              *list_item)
+{
+  GtkWidget *label;
+  gpointer item;
+
+  item = gtk_tree_list_row_get_item (gtk_list_item_get_item (list_item));
+  label = gtk_list_item_get_child (list_item);
+
+  gtk_label_set_label (GTK_LABEL (label), gtk_inspector_get_object_name (item));
+
+  g_object_unref (item);
+}
+
+static void
+setup_label_cb (GtkSignalListItemFactory *factory,
+                GtkListItem              *list_item)
+{
+  GtkWidget *label;
+
+  label = gtk_label_new (NULL);
+  gtk_label_set_width_chars (GTK_LABEL (label), 15);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_list_item_set_child (list_item, label);
+}
+
+static void
+bind_label_cb (GtkSignalListItemFactory *factory,
+               GtkListItem              *list_item)
+{
+  GtkWidget *label;
+  gpointer item;
+  GBinding *binding = NULL;
+
+  item = gtk_tree_list_row_get_item (gtk_list_item_get_item (list_item));
+  label = gtk_list_item_get_child (list_item);
 
-  /* 3rd column: label */
-  child = gtk_label_new (NULL);
   if (GTK_IS_LABEL (item))
-    g_object_bind_property (item, "label", child, "label", G_BINDING_SYNC_CREATE);
+    binding = g_object_bind_property (item, "label", label, "label", G_BINDING_SYNC_CREATE);
   else if (GTK_IS_BUTTON (item))
-    g_object_bind_property (item, "label", child, "label", G_BINDING_SYNC_CREATE);
+    binding = g_object_bind_property (item, "label", label, "label", G_BINDING_SYNC_CREATE);
   else if (GTK_IS_WINDOW (item))
-    g_object_bind_property (item, "title", child, "label", G_BINDING_SYNC_CREATE);
+    binding = g_object_bind_property (item, "title", label, "label", G_BINDING_SYNC_CREATE);
   else if (GTK_IS_TREE_VIEW_COLUMN (item))
-    g_object_bind_property (item, "title", child, "label", G_BINDING_SYNC_CREATE);
-  gtk_label_set_width_chars (GTK_LABEL (child), 15);
-  gtk_label_set_xalign (GTK_LABEL (child), 0.0);
-  gtk_size_group_add_widget (wt->priv->label_size_group, child);
-  gtk_box_append (GTK_BOX (box), child);
+    binding = g_object_bind_property (item, "title", label, "label", G_BINDING_SYNC_CREATE);
+  else
+    gtk_label_set_label (GTK_LABEL (label), NULL);
 
   g_object_unref (item);
 
-  return row;
+  if (binding)
+    g_object_set_data (G_OBJECT (label), "binding", binding);
+}
+
+static void
+unbind_label_cb (GtkSignalListItemFactory *factory,
+                 GtkListItem              *list_item)
+{
+  GtkWidget *label;
+  GBinding *binding;
+
+  label = gtk_list_item_get_child (list_item);
+  binding = g_object_steal_data (G_OBJECT (label), "binding");
+  if (binding)
+    g_binding_unbind (binding);
 }
 
 static GListModel *
@@ -1091,6 +1163,7 @@ toplevel_filter_func (gpointer item,
 static GListModel *
 create_root_model (GdkDisplay *display)
 {
+  GtkFilter *custom_filter;
   GtkFilterListModel *filter;
   GtkFlattenListModel *flatten;
   GListStore *list, *special;
@@ -1107,9 +1180,9 @@ create_root_model (GdkDisplay *display)
   g_object_unref (special);
 
   filter = gtk_filter_list_model_new_for_type (G_TYPE_OBJECT);
-  gtk_filter_list_model_set_filter_func (filter, 
-                                         toplevel_filter_func,
+  custom_filter = gtk_custom_filter_new (toplevel_filter_func,
                                          display, NULL);
+  gtk_filter_list_model_set_filter (filter, custom_filter);
   gtk_filter_list_model_set_model (filter, gtk_window_get_toplevels ());
   g_list_store_append (list, filter);
   g_object_unref (filter);
@@ -1135,6 +1208,7 @@ gtk_inspector_object_tree_dispose (GObject *object)
   GtkInspectorObjectTree *wt = GTK_INSPECTOR_OBJECT_TREE (object);
 
   g_clear_object (&wt->priv->tree_model);
+  g_clear_object (&wt->priv->selection);
 
   G_OBJECT_CLASS (gtk_inspector_object_tree_parent_class)->dispose (object);
 }
@@ -1172,14 +1246,19 @@ gtk_inspector_object_tree_class_init (GtkInspectorObjectTreeClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorObjectTree, list);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorObjectTree, search_bar);
   gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorObjectTree, search_entry);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorObjectTree, type_size_group);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorObjectTree, name_size_group);
-  gtk_widget_class_bind_template_child_private (widget_class, GtkInspectorObjectTree, label_size_group);
   gtk_widget_class_bind_template_callback (widget_class, on_search_changed);
   gtk_widget_class_bind_template_callback (widget_class, on_row_activated);
   gtk_widget_class_bind_template_callback (widget_class, next_match);
   gtk_widget_class_bind_template_callback (widget_class, previous_match);
   gtk_widget_class_bind_template_callback (widget_class, stop_search);
+  gtk_widget_class_bind_template_callback (widget_class, setup_type_cb);
+  gtk_widget_class_bind_template_callback (widget_class, bind_type_cb);
+  gtk_widget_class_bind_template_callback (widget_class, unbind_type_cb);
+  gtk_widget_class_bind_template_callback (widget_class, setup_name_cb);
+  gtk_widget_class_bind_template_callback (widget_class, bind_name_cb);
+  gtk_widget_class_bind_template_callback (widget_class, setup_label_cb);
+  gtk_widget_class_bind_template_callback (widget_class, bind_label_cb);
+  gtk_widget_class_bind_template_callback (widget_class, unbind_label_cb);
 }
 
 static guint
@@ -1236,16 +1315,13 @@ gtk_inspector_object_tree_select_object (GtkInspectorObjectTree *wt,
                                          GObject                *object)
 {
   GtkTreeListRow *row_item;
-  GtkListBoxRow *row_widget;
 
   row_item = find_and_expand_object (wt->priv->tree_model, object);
   if (row_item == NULL)
     return;
 
-  row_widget = gtk_list_box_get_row_at_index (wt->priv->list,
-                                              gtk_tree_list_row_get_position (row_item));
-  g_return_if_fail (row_widget != NULL);
-  gtk_list_box_select_row (wt->priv->list, row_widget);
+  gtk_single_selection_set_selected (wt->priv->selection,
+                                     gtk_tree_list_row_get_position (row_item));
   g_signal_emit (wt, signals[OBJECT_SELECTED], 0, object); // FIXME
   g_object_unref (row_item);
 }
@@ -1263,11 +1339,9 @@ gtk_inspector_object_tree_set_display (GtkInspectorObjectTree *wt,
                                                   create_model_for_object,
                                                   NULL,
                                                   NULL);
+  wt->priv->selection = gtk_single_selection_new (G_LIST_MODEL (wt->priv->tree_model));
   g_object_unref (root_model);
 
-  gtk_list_box_bind_model (wt->priv->list,
-                           G_LIST_MODEL (wt->priv->tree_model),
-                           gtk_inspector_object_tree_create_list_widget,
-                           wt,
-                           NULL);
+  gtk_column_view_set_model (GTK_COLUMN_VIEW (wt->priv->list),
+                             G_LIST_MODEL (wt->priv->selection));
 }
