@@ -79,7 +79,7 @@ create_row (const gchar *text)
   row_content = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
 
   label = gtk_label_new (text);
-  gtk_container_add (GTK_CONTAINER (row_content), label);
+  gtk_box_append (GTK_BOX (row_content), label);
 
   return row_content;
 }
@@ -101,7 +101,7 @@ new_window (GApplication *app)
 
   /* widget creation */
   grid = gtk_grid_new ();
-  gtk_container_add (GTK_CONTAINER (window), grid);
+  gtk_window_set_child (GTK_WINDOW (window), grid);
   sw = gtk_scrolled_window_new (NULL, NULL);
   gtk_widget_set_hexpand (GTK_WIDGET (sw), true);
   gtk_widget_set_vexpand (GTK_WIDGET (sw), true);
@@ -110,7 +110,7 @@ new_window (GApplication *app)
   list = gtk_list_box_new ();
   gtk_list_box_set_selection_mode (GTK_LIST_BOX (list), GTK_SELECTION_NONE);
   gtk_list_box_set_header_func (GTK_LIST_BOX (list), add_separator, NULL, NULL);
-  gtk_container_add (GTK_CONTAINER (sw), list);
+  gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (sw), list);
 
   label = gtk_label_new ("No row activated");
   gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
