@@ -38,7 +38,6 @@ int main (int argc, char **argv)
 	GtkWidget *grid;
 	GtkWidget *entry;
 	GtkWidget *label;
-	GtkWidget *check;
 	GtkWidget *combo;
 	guint i;
 	guint row = 0;
@@ -52,11 +51,10 @@ int main (int argc, char **argv)
 	grid = gtk_grid_new ();
 	gtk_grid_set_row_spacing (GTK_GRID (grid), 12);
 	gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
-	gtk_container_add (GTK_CONTAINER (window), grid);
+	gtk_window_set_child (GTK_WINDOW (window), grid);
 
 	/* horizontal alignment */
 	label = gtk_label_new ("Horizontal Alignment:");
-	gtk_widget_show (label);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, row++, 1, 1);
 
 	combo = gtk_combo_box_text_new ();
@@ -66,14 +64,12 @@ int main (int argc, char **argv)
 	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), "Center");
 	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), "Baseline");
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), INITIAL_HALIGN);
-	gtk_widget_show (combo);
 	gtk_grid_attach_next_to (GTK_GRID (grid), combo, label, GTK_POS_RIGHT, 1, 1);
 	g_signal_connect (G_OBJECT (combo), "changed",
 			  G_CALLBACK (horizontal_alignment_changed), menubuttons);
 
 	/* vertical alignment */
 	label = gtk_label_new ("Vertical Alignment:");
-	gtk_widget_show (label);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, row++, 1, 1);
 
 	combo = gtk_combo_box_text_new ();
@@ -83,7 +79,6 @@ int main (int argc, char **argv)
 	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), "Center");
 	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), "Baseline");
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), INITIAL_HALIGN);
-	gtk_widget_show (combo);
 	gtk_grid_attach_next_to (GTK_GRID (grid), combo, label, GTK_POS_RIGHT, 1, 1);
 	g_signal_connect (G_OBJECT (combo), "changed",
 			  G_CALLBACK (vertical_alignment_changed), menubuttons);
@@ -97,10 +92,6 @@ int main (int argc, char **argv)
 	gtk_grid_attach_next_to (GTK_GRID (grid), button, entry, GTK_POS_RIGHT, 1, 1);
 	menubuttons = g_list_prepend (menubuttons, button);
 
-        check = gtk_check_button_new_with_label ("Popover");
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
-	gtk_grid_attach (GTK_GRID (grid), check, 0, row, 1, 1);
-
 	/* Button with GMenuModel */
 	menu = g_menu_new ();
 	for (i = 5; i > 0; i--) {
@@ -116,7 +107,6 @@ int main (int argc, char **argv)
 	}
 
 	button = gtk_menu_button_new ();
-        g_object_bind_property (check, "active", button, "use-popover", G_BINDING_SYNC_CREATE);
 
 	gtk_widget_set_halign (button, GTK_ALIGN_START);
 	menubuttons = g_list_prepend (menubuttons, button);
