@@ -27,22 +27,19 @@ selectable_row_init (SelectableRow *row)
   gtk_widget_set_margin_top (row->check, 10);
   gtk_widget_set_margin_bottom (row->check, 10);
 
-  gtk_widget_show (row->box);
-  gtk_widget_show (row->check);
-
-  gtk_container_add (GTK_CONTAINER (row), row->box);
-  gtk_container_add (GTK_CONTAINER (row->box), row->revealer);
-  gtk_container_add (GTK_CONTAINER (row->revealer), row->check);
+  gtk_list_box_row_set_child (GTK_LIST_BOX_ROW (row), row->box);
+  gtk_box_append (GTK_BOX (row->box), row->revealer);
+  gtk_revealer_set_child (GTK_REVEALER (row->revealer), row->check);
 }
 
 static void
 selectable_row_add (SelectableRow *row, GtkWidget *child)
 {
-  gtk_container_add (GTK_CONTAINER (row->box), child);
+  gtk_box_append (GTK_BOX (row->box), child);
 }
 
 static void
-update_selectable (GtkWidget *widget, gpointer data)
+update_selectable (GtkWidget *widget)
 {
   SelectableRow *row = (SelectableRow *)widget;
   GtkListBox *list;
@@ -56,7 +53,7 @@ update_selectable (GtkWidget *widget, gpointer data)
 }
 
 static void
-update_selected (GtkWidget *widget, gpointer data)
+update_selected (GtkWidget *widget)
 {
   SelectableRow *row = (SelectableRow *)widget;
 
@@ -106,6 +103,7 @@ selection_mode_enter (GtkButton *button, GtkBuilder *builder)
   GtkWidget *cancelbutton;
   GtkWidget *selectbutton;
   GtkWidget *titlestack;
+  GtkWidget *child;
 
   header = GTK_WIDGET (gtk_builder_get_object (builder, "header"));
   list = GTK_WIDGET (gtk_builder_get_object (builder, "list"));
@@ -123,7 +121,10 @@ selection_mode_enter (GtkButton *button, GtkBuilder *builder)
 
   gtk_list_box_set_activate_on_single_click (GTK_LIST_BOX (list), FALSE);
   gtk_list_box_set_selection_mode (GTK_LIST_BOX (list), GTK_SELECTION_MULTIPLE);
-  gtk_container_forall (GTK_CONTAINER (list), update_selectable, NULL);
+  for (child = gtk_widget_get_first_child (list);
+       child != NULL;
+       child = gtk_widget_get_next_sibling (child))
+    update_selectable (child);
 }
 
 static void
@@ -135,6 +136,7 @@ selection_mode_leave (GtkButton *button, GtkBuilder *builder)
   GtkWidget *cancelbutton;
   GtkWidget *selectbutton;
   GtkWidget *titlestack;
+  GtkWidget *child;
 
   header = GTK_WIDGET (gtk_builder_get_object (builder, "header"));
   list = GTK_WIDGET (gtk_builder_get_object (builder, "list"));
@@ -152,7 +154,10 @@ selection_mode_leave (GtkButton *button, GtkBuilder *builder)
 
   gtk_list_box_set_activate_on_single_click (GTK_LIST_BOX (list), TRUE);
   gtk_list_box_set_selection_mode (GTK_LIST_BOX (list), GTK_SELECTION_NONE);
-  gtk_container_forall (GTK_CONTAINER (list), update_selectable, NULL);
+  for (child = gtk_widget_get_first_child (list);
+       child != NULL;
+       child = gtk_widget_get_next_sibling (child))
+    update_selectable (child);
 }
 
 static void
@@ -170,7 +175,12 @@ select_none (GAction *action, GVariant *param, GtkWidget *list)
 static void
 selected_rows_changed (GtkListBox *list)
 {
-  gtk_container_forall (GTK_CONTAINER (list), update_selected, NULL);
+  GtkWidget *child;
+
+  for (child = gtk_widget_get_first_child (GTK_WIDGET (list));
+       child != NULL;
+       child = gtk_widget_get_next_sibling (child))
+    update_selected (child);
 }
 
 int
