@@ -210,7 +210,7 @@ create_control (GtkWidget *box, gint number, gint cntl, CallbackData *data)
 
   checkbutton = gtk_check_button_new_with_label (name);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), cntl == CNTL_FIXED);
-  gtk_container_add (GTK_CONTAINER (box), checkbutton);
+  gtk_box_append (GTK_BOX (box), checkbutton);
 
   g_signal_connect (G_OBJECT (checkbutton), "toggled", callback, data);
   g_free (name);
@@ -252,14 +252,14 @@ main (gint argc, gchar **argv)
   g_signal_connect (window, "destroy", G_CALLBACK (quit_cb), &done);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
+  gtk_window_set_child (GTK_WINDOW (window), vbox);
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_has_frame (GTK_SCROLLED_WINDOW (scrolled_window), TRUE);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_widget_set_vexpand (scrolled_window, TRUE);
-  gtk_container_add (GTK_CONTAINER (vbox), scrolled_window);
+  gtk_box_append (GTK_BOX (vbox), scrolled_window);
 
   tree_model = create_model ();
   tree_view = gtk_tree_view_new_with_model (tree_model);
@@ -358,17 +358,17 @@ main (gint argc, gchar **argv)
 					       "value", PROGRESS_COLUMN,
 					       NULL);
 
-  gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
+  gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled_window), tree_view);
 
   gtk_window_set_default_size (GTK_WINDOW (window),
 			       800, 250);
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_container_add (GTK_CONTAINER (vbox), hbox);
+  gtk_box_append (GTK_BOX (vbox), hbox);
 
   /* Alignment controls */
   cntl_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-  gtk_container_add (GTK_CONTAINER (hbox), cntl_vbox);
+  gtk_box_append (GTK_BOX (hbox), cntl_vbox);
 
   create_control (cntl_vbox, 1, CNTL_ALIGN, &callback[0]);
   create_control (cntl_vbox, 2, CNTL_ALIGN, &callback[1]);
@@ -377,7 +377,7 @@ main (gint argc, gchar **argv)
 
   /* Expand controls */
   cntl_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-  gtk_container_add (GTK_CONTAINER (hbox), cntl_vbox);
+  gtk_box_append (GTK_BOX (hbox), cntl_vbox);
 
   create_control (cntl_vbox, 1, CNTL_EXPAND, &callback[0]);
   create_control (cntl_vbox, 2, CNTL_EXPAND, &callback[1]);
@@ -386,7 +386,7 @@ main (gint argc, gchar **argv)
 
   /* Fixed controls */
   cntl_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-  gtk_container_add (GTK_CONTAINER (hbox), cntl_vbox);
+  gtk_box_append (GTK_BOX (hbox), cntl_vbox);
 
   create_control (cntl_vbox, 1, CNTL_FIXED, &callback[0]);
   create_control (cntl_vbox, 2, CNTL_FIXED, &callback[1]);
