@@ -122,20 +122,20 @@ main (int argc, char *argv[])
   gtk_window_set_title (GTK_WINDOW (window), "Words, words, words - Window 1");
   g_signal_connect (window, "destroy", G_CALLBACK (quit_cb), &done);
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
-  gtk_container_add (GTK_CONTAINER (vbox), gtk_label_new ("Jonathan and Kristian's list of cool words. (And Anders' cool list of numbers) \n\nThis is just a GtkTreeStore"));
-  gtk_container_add (GTK_CONTAINER (window), vbox);
+  gtk_box_append (GTK_BOX (vbox), gtk_label_new ("Jonathan and Kristian's list of cool words. (And Anders' cool list of numbers) \n\nThis is just a GtkTreeStore"));
+  gtk_window_set_child (GTK_WINDOW (window), vbox);
 
   entry = gtk_entry_new ();
-  gtk_container_add (GTK_CONTAINER (vbox), entry);
+  gtk_box_append (GTK_BOX (vbox), entry);
 
   button = gtk_button_new_with_label ("Switch search method");
-  gtk_container_add (GTK_CONTAINER (vbox), button);
+  gtk_box_append (GTK_BOX (vbox), button);
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_has_frame (GTK_SCROLLED_WINDOW (scrolled_window), TRUE);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_widget_set_vexpand (scrolled_window, TRUE);
-  gtk_container_add (GTK_CONTAINER (vbox), scrolled_window);
+  gtk_box_append (GTK_BOX (vbox), scrolled_window);
 
   model = gtk_tree_store_new (NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
 
@@ -235,7 +235,7 @@ main (int argc, char *argv[])
 					WORD_COLUMN_1,
 					GTK_SORT_ASCENDING);*/
 
-  gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
+  gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled_window), tree_view);
   gtk_window_set_default_size (GTK_WINDOW (window), 400, 400);
   gtk_widget_show (window);
 
@@ -250,9 +250,9 @@ main (int argc, char *argv[])
 			    "Words, words, words - window 2");
       g_signal_connect (window2, "destroy", G_CALLBACK (quit_cb), &done);
       vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
-      gtk_container_add (GTK_CONTAINER (vbox2), 
+      gtk_box_append (GTK_BOX (vbox2), 
 			  gtk_label_new ("Jonathan and Kristian's list of words.\n\nA GtkTreeModelSort wrapping the GtkTreeStore of window 1"));
-      gtk_container_add (GTK_CONTAINER (window2), vbox2);
+      gtk_window_set_child (GTK_WINDOW (window2), vbox2);
       
       scrolled_window2 = gtk_scrolled_window_new (NULL, NULL);
       gtk_scrolled_window_set_has_frame (GTK_SCROLLED_WINDOW (scrolled_window2), TRUE);
@@ -260,7 +260,7 @@ main (int argc, char *argv[])
 				      GTK_POLICY_AUTOMATIC,
 				      GTK_POLICY_AUTOMATIC);
       gtk_widget_set_vexpand (scrolled_window2, TRUE);
-      gtk_container_add (GTK_CONTAINER (vbox2), scrolled_window2);
+      gtk_box_append (GTK_BOX (vbox2), scrolled_window2);
 
 
       tree_view2 = gtk_tree_view_new_with_model (smodel);
@@ -299,9 +299,8 @@ main (int argc, char *argv[])
       gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (smodel),
 					    WORD_COLUMN_1,
 					    GTK_SORT_DESCENDING);*/
-      
-      
-      gtk_container_add (GTK_CONTAINER (scrolled_window2), tree_view2);
+
+      gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled_window2), tree_view2);
       gtk_window_set_default_size (GTK_WINDOW (window2), 400, 400);
       gtk_widget_show (window2);
     }
@@ -318,9 +317,9 @@ main (int argc, char *argv[])
 			    "Words, words, words - Window 3");
       g_signal_connect (window3, "destroy", G_CALLBACK (quit_cb), &done);
       vbox3 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
-      gtk_container_add (GTK_CONTAINER (vbox3), 
+      gtk_box_append (GTK_BOX (vbox3), 
 			  gtk_label_new ("Jonathan and Kristian's list of words.\n\nA GtkTreeModelSort wrapping the GtkTreeModelSort of window 2"));
-      gtk_container_add (GTK_CONTAINER (window3), vbox3);
+      gtk_window_set_child (GTK_WINDOW (window3), vbox3);
       
       scrolled_window3 = gtk_scrolled_window_new (NULL, NULL);
       gtk_scrolled_window_set_has_frame (GTK_SCROLLED_WINDOW (scrolled_window3), TRUE);
@@ -328,7 +327,7 @@ main (int argc, char *argv[])
 				      GTK_POLICY_AUTOMATIC,
 				      GTK_POLICY_AUTOMATIC);
       gtk_widget_set_vexpand (scrolled_window3, TRUE);
-      gtk_container_add (GTK_CONTAINER (vbox3), scrolled_window3);
+      gtk_box_append (GTK_BOX (vbox3), scrolled_window3);
 
 
       tree_view3 = gtk_tree_view_new_with_model (ssmodel);
@@ -367,8 +366,8 @@ main (int argc, char *argv[])
       gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (ssmodel),
 					    WORD_COLUMN_1,
 					    GTK_SORT_ASCENDING);*/
-      
-      gtk_container_add (GTK_CONTAINER (scrolled_window3), tree_view3);
+
+      gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled_window3), tree_view3);
       gtk_window_set_default_size (GTK_WINDOW (window3), 400, 400);
       gtk_widget_show (window3);
     }
