@@ -424,8 +424,8 @@ test_phases (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -457,7 +457,7 @@ test_phases (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -476,8 +476,8 @@ test_mixed (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -516,7 +516,7 @@ test_mixed (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -535,8 +535,8 @@ test_early_exit (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -571,7 +571,7 @@ test_early_exit (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -590,8 +590,8 @@ test_claim_capture (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -618,7 +618,7 @@ test_claim_capture (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -637,8 +637,8 @@ test_claim_target (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -665,7 +665,7 @@ test_claim_target (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -684,8 +684,8 @@ test_claim_bubble (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -718,7 +718,7 @@ test_claim_bubble (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -738,8 +738,8 @@ test_early_claim_capture (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -775,7 +775,7 @@ test_early_claim_capture (void)
   point_release (&mouse_state, 1);
 
   g_string_free (str, TRUE);
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -795,8 +795,8 @@ test_late_claim_capture (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -834,7 +834,7 @@ test_late_claim_capture (void)
   point_release (&mouse_state, 1);
 
   g_string_free (str, TRUE);
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -854,8 +854,8 @@ test_group (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -886,7 +886,7 @@ test_group (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -905,8 +905,8 @@ test_gestures_outside_grab (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -945,8 +945,8 @@ test_gestures_outside_grab (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
-  gtk_widget_destroy (D);
+  gtk_window_destroy (A);
+  gtk_window_destroy (D);
 }
 
 static void
@@ -965,8 +965,8 @@ test_gestures_inside_grab (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -1006,7 +1006,7 @@ test_gestures_inside_grab (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -1025,8 +1025,8 @@ test_multitouch_on_single (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -1057,7 +1057,7 @@ test_multitouch_on_single (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -1076,8 +1076,8 @@ test_multitouch_activation (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -1132,7 +1132,7 @@ test_multitouch_activation (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 static void
@@ -1152,8 +1152,8 @@ test_multitouch_interaction (void)
   gtk_widget_set_vexpand (C, TRUE);
   gtk_widget_set_name (C, "C");
 
-  gtk_container_add (GTK_CONTAINER (A), B);
-  gtk_container_add (GTK_CONTAINER (B), C);
+  gtk_box_append (GTK_BOX (A), B);
+  gtk_box_append (GTK_BOX (B), C);
 
   gtk_widget_show (A);
 
@@ -1225,7 +1225,7 @@ test_multitouch_interaction (void)
 
   g_string_free (str, TRUE);
 
-  gtk_widget_destroy (A);
+  gtk_window_destroy (A);
 }
 
 int
